unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    function InitVertexBuffer: boolean;
    procedure SetupMatrices;
    procedure SetupMaterialAndLight;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

var
  VertexBuffer : IDirect3DVertexBuffer8 = NIL;
  Texture      : IDirect3DTexture8 = NIL;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
 // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    Position : TD3DXVector3; // Position
                    Normal   : TD3DXVector3; // Surface Normal
                    U, V     : single; // Texture coordinates.
                  end;
const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1;

  // Texture filtering.
  USE_TEXTURE_FILTERING = false;
  TEXTFILENAME = 'GRID.PNG';

procedure TForm1.SetupMaterialAndLight;
var _mat : TD3DMaterial8;
    _lit : TD3DLight8;
    _tim : single;
    _vecDir : TD3DXVector3;
begin
  // Set Material
  FillChar(_mat, sizeof(_mat), 0 );
  with _mat do
    begin
      Ambient.r := 1.0;
      Ambient.g := 1.0;
      Ambient.b := 1.0;
      Ambient.a := 1.0;
      Diffuse  := Ambient;
      Specular.r := 1.0;
      Specular.g := 0.8;
      Specular.b := 0.8;
      Specular.a := 1.0;
      Power := 20.0;
      OmegaScreen1.Device.SetMaterial(_mat);
    end;

  // Set Directional Light
  FillChar(_lit, sizeof(_lit), 0 );
  with _lit do
    begin
      _Type := D3DLIGHT_DIRECTIONAL;
      Diffuse.r := 1.0;
      Diffuse.g := 1.0;
      Diffuse.b := 1.0;
      Specular.r := 1.0;
      Specular.g := 1.0;
      Specular.b := 1.0;
      _tim := GetTickCount / 360;
      _vecDir := D3DXVECTOR3(cos(_tim), 0.0, sin(_tim));
      D3DXVec3Normalize(Direction, _vecDir);
      Range   := 1000.0;
      OmegaScreen1.Device.SetLight(0, _lit);
      OmegaScreen1.Device.LightEnable(0, true);
    end;
end;

function GetD3DXVector3(_x, _y, _z: single): TD3DXVector3;
begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

function GetCustomVertex(_px, _py, _pz, _nx, _ny, _nz,
                         _u, _v: single): TCustomVertex;

begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
     Normal.x := _nx;
     Normal.y := _ny;
     Normal.z := _nz;
     U := _u;
     V := _v;
   end;
end;

procedure TForm1.SetupMatrices;
var _v1, _v2, _v3 : TD3DXVector3;
begin
  // World.
  D3DXMatrixRotationX(matWorld, sin(GetTickCount/1500) * 1.2);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);

  // View.
  _v1 := GetD3DXVector3( 0.0, 0.0, -5.0);
  _v2 := GetD3DXVector3( 0.0, 0.0,  0.0);
  _v3 := GetD3DXVector3( 0.0, 1.0,  0.0);
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
end;

function TForm1.InitVertexBuffer: boolean;
var _vb  : array[0..99] of TCustomVertex;
    _cnt : longint;
    _ptr : pByte;
    _theta : single;

begin
  Result := false;

  // Initialize the vertices.
  for _cnt := 0 to 49 do
    begin
      _theta := (2 * PI * _cnt) /49;
      _vb[2 * _cnt] := GetCustomVertex(sin(_theta), -1.0, cos(_theta),
                                       sin(_theta),  0.0, cos(_theta),
                                       _cnt/49, 1);
      _vb[2 * _cnt + 1] := GetCustomVertex(sin(_theta),  1.0, cos(_theta),
                                           sin(_theta),  0.0, cos(_theta),
                                           _cnt/49, 0);
    end;

  // Create a vertex buffer.
  if failed(OmegaScreen1.Device.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_MANAGED,
                                       VertexBuffer)) then exit;

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer.Unlock;

  // All right
  Result := true;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  InitVertexBuffer;
  // Set Material And Light
  SetupMaterialAndLight;
  D3DXCreateTextureFromFileExA(OmegaScreen1.Device, TEXTFILENAME,
                                0, 0, 0, 0, _d3dformat(0), D3DPOOL_MANAGED,
                                D3DX_FILTER_NONE, D3DX_FILTER_NONE,
                                $FF00FF00, NIL, NIL, Texture);
  // Disable culling.
  SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
  // Enable lights.
  SetRenderState(D3DRS_LIGHTING, 1);
  // Set ambiental light.
  SetRenderState(D3DRS_AMBIENT, $202020);
  // Enable spelular lights.
  SetRenderState(D3DRS_SPECULARENABLE, 1);
  // Set texture filtering
  if USE_TEXTURE_FILTERING then
    begin
      SetTextureStageState(0, D3DTSS_MAGFILTER, ord(D3DTEXF_LINEAR));
      SetTextureStageState(0, D3DTSS_MINFILTER, ord(D3DTEXF_LINEAR));
    end;
  // Set texture
  SetTexture(0, Texture);
  // Enable Aplha Test (colorkeying)
  SetRenderState(D3DRS_ALPHAREF, 1);
  SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
  SetRenderState(D3DRS_ALPHATESTENABLE, 1);
  // Set texture stages (colorkeying)
  SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
  SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1);
 end;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var _mat : TD3DXMatrix;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
  OmegaScreen1.ToggleFullScreen;
 OmegaScreen1.BeginRender;
 OmegaScreen1.ClearScreen(55, 0, 0);
 with OmegaScreen1.Device do
 begin
  // Set Material and Light.
  SetupMaterialAndLight;
  // Set Transform states.
  SetupMatrices;
  // Set the stream source.
  SetStreamSource(0, VertexBuffer, sizeof(TCustomVertex));
  // Set the vertex shader.
  SetVertexShader(D3DFVF_CUSTOMVERTEX);
  SetTexture(0, Texture);
  // Set Material and Light.
  SetupMaterialAndLight;
  // Set Transform states.
  SetupMatrices;
  // Set the stream source.
  SetStreamSource(0, VertexBuffer, sizeof(TCustomVertex));
  // Set the vertex shader.
  SetVertexShader(D3DFVF_CUSTOMVERTEX);
  // Render the vertices.
  DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 98);
 end;

 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');

 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 VertexBuffer := NIL;
 Texture := NIL; // Free Texture;
end;

end.



