unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    OpenDialog1: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    function InitGeometry: boolean;
    procedure SetLights;
    procedure SetMatrices;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

var
  ZDist : single; // Camera Z distance.
  CDist : single; // Z-Clipping distance.

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

  MeshFileName : string;
  g_dwNumMaterials : dWord;
  g_pMeshMaterials : array of TD3DMaterial8 = NIL;
  g_pMeshTextures  : array of IDirect3DTexture8 = NIL;
  g_pMesh : ID3DXMesh = NIL;

const
  SCREENWIDTH  = 800;
  SCREENHEIGHT = 600;
  // Texture filtering.
  USE_TEXTURE_FILTERING = true;
  // Specular lights.
  USE_SPECULAR_LIGHTS = false;

function TForm1.InitGeometry: boolean; // Tutorial 6 - Step 1
var _pAdjacency, _pD3DXMtrlBuffer : ID3DXBuffer;
    _d3dxMaterials : PD3DXMaterial;
    _i : longint;
begin
  Result := false;
  // Load mesh.
  _pAdjacency := NIL;
  _pD3DXMtrlBuffer := NIL;
  if failed(D3DXLoadMeshFromX(pChar(MeshFileName), D3DXMESH_SYSTEMMEM,
                              OmegaScreen1.Device, @_pAdjacency, @_pD3DXMtrlBuffer,
                              @g_dwNumMaterials, g_pMesh)) then exit;
  _pAdjacency := NIL;
  if g_dwNumMaterials = 0 then exit;
  // After loading the mesh object and material information,
  // we need to extract the material properties and texture
  // names from the material buffer.
  _d3dxMaterials := _pD3DXMtrlBuffer.GetBufferPointer;

  // Creates new mesh and texture objects based on the total
  // number of materials for the mesh.
  try
    SetLength(g_pMeshMaterials, g_dwNumMaterials);
    SetLength(g_pMeshTextures, g_dwNumMaterials);
  except
    exit;
  end;

  // Load materials and textures.
  for _i := 0 to g_dwNumMaterials - 1 do
    begin
      // Copy the material
      g_pMeshMaterials[_i] := _d3dxMaterials^.MatD3D;

      // Set the ambient color for the material (D3DX does not do this)
      g_pMeshMaterials[_i].Ambient := g_pMeshMaterials[_i].Diffuse;

      // Create the texture.
      g_pMeshTextures[_i] := NIL;
//      _d3dxMaterials^.pTextureFilename:='a.png';
      if (_d3dxMaterials^.pTextureFilename = NIL) or
         (failed(D3DXCreateTextureFromFile(OmegaScreen1.Device,
                 _d3dxMaterials^.pTextureFilename,
                 g_pMeshTextures[_i])))
        then g_pMeshTextures[_i] := NIL;

      // Next
      inc(_d3dxMaterials);
    end;

  // Done with the material buffer
  _pD3DXMtrlBuffer := NIL;

  // All right.
  Result := true;
end;

procedure TForm1.FormCreate(Sender: TObject);
var _cnt  : longint;
    _data : PD3DXVector3;
    _vb : IDirect3DVertexBuffer8;
    _x, _z : single;
begin
 if OpenDialog1.Execute then
  MeshFileName := OpenDialog1.Filename else
 begin
  Application.Terminate;
  exit;
 end;
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  InitGeometry;
  // Get camera distance.
  ZDist := 0;
  g_pMesh.GetVertexBuffer(_vb);
  _vb.Lock(0, 0, pByte(_data), D3DLOCK_READONLY);
  for _cnt := 0 to g_pMesh.GetNumVertices - 1 do
    begin
      if _data.x < 0 then _x := _data.x * -1 else _x := _data.x;
      if _data.z < 0 then _z := _data.z * -1 else _z := _data.z;
      if ZDist < _x then ZDist := _x;
      if ZDist < _z then ZDist := _z;
      inc(_data);
    end;
  _vb.Unlock;
  _vb := NIL;
  ZDist := ZDist * 3;
  CDist := ZDist * 2;
  // Set ambiental light.
  SetRenderState(D3DRS_AMBIENT, $111111);
  // Enable dithering
  SetRenderState(D3DRS_DITHERENABLE, 1);
  // Enable Specular Lights
  if USE_SPECULAR_LIGHTS
    then SetRenderState(D3DRS_SPECULARENABLE, 1);
  // Set texture filtering
  if USE_TEXTURE_FILTERING then
    begin
      SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
      SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    end;
 end;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var _i : longint;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
  OmegaScreen1.ToggleFullScreen;

 OmegaScreen1.BeginRender;
 OmegaScreen1.ClearScreen(55, 0, 0);
 with OmegaScreen1.Device do
 begin
  SetMatrices;
  SetLights;
  // Draw mesh - // Tutorial 6 - Step 2
  for _i := 0 to g_dwNumMaterials - 1 do
    begin
      // Set the material for the subset.
      SetMaterial(g_pMeshMaterials[_i]);
      // Set the texture for the subset.
      SetTexture(0, g_pMeshTextures[_i]);
      // Draw subset.
      g_pMesh.DrawSubset(_i);
    end;
 end;

 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');

 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

procedure TForm1.SetMatrices;
var _v1, _v2, _v3 : TD3DXVector3;
begin
  // World.
  D3DXMatrixRotationY(matWorld, sin(GetTickCount/2500) * Pi);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);
  // View.
  _v1 := D3DXVector3(  0.0,  0.0,   ZDist);
  _v2 := D3DXVector3(  0.0,  0.0,   0.0);
  _v3 := D3DXVector3(  0.0,  1.0,   0.0);
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);
  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, SCREENWIDTH/SCREENHEIGHT,
                             1.0, CDist);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
end;

procedure TForm1.SetLights;
var _lit : TD3DLight8;
    _tim : single;
    _vecDir : TD3DXVector3;
begin
  // Set Directional Light
  FillChar(_lit, sizeof(_lit), 0 );
  with _lit do
    begin
      _Type := D3DLIGHT_DIRECTIONAL;
      Diffuse.r := 1.0;
      Diffuse.g := 1.0;
      Diffuse.b := 1.0;
      Specular.r := 0.5;
      Specular.g := 0.5;
      Specular.b := 0.5;
      _tim := GetTickCount / 1000;
      _vecDir := D3DXVECTOR3(cos(_tim), 0.0, sin(_tim));
      D3DXVec3Normalize(Direction, _vecDir);
      Range   := 1.0;
      OmegaScreen1.Device.SetLight(0, _lit);
      OmegaScreen1.Device.LightEnable(0, true);
    end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var _i : longint;
begin
  // Free materials.
  if (g_pMeshMaterials <> NIL) then g_pMeshMaterials := NIL;
  // Free textures.
  if (g_pMeshMaterials <> NIL) and (g_dwNumMaterials > 0) then
    for _i := 0 to g_dwNumMaterials - 1 do
      if g_pMeshTextures[_i] <> NIL
        then g_pMeshTextures[_i] := NIL;
  g_pMeshTextures := NIL;
  // Free mesh.
  if (g_pMesh <> NIL) then g_pMesh := NIL;
end;

end.



