unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    procedure RenderScene;  // Render the scene
    procedure RenderMirror; // Render the scene in the mirror
    procedure FrameMove;
    function InitVertexBuffer: boolean;
    procedure InitMatrices;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

var
  VertexBuffer : IDirect3DVertexBuffer8;
  MeshMirror   : IDirect3DVertexBuffer8;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection
  matObj   : TD3DXMatrix; // Object matrix.

  // View.
  vEyePt, vLookatPt, vUpVec : TD3DXVector3;

type
  // The structure of the custom vertex.
  TCustomVertex = record
                    X, Y, Z : single; // Position for the vertex.
                    DColor  : dWord;  // Diffuse color.
                  end;

  TMirrorVertex = record
                    Position : TD3DXVector3;
                    Normal   : TD3DXVector3;
                    DColor   : dWord; // Diffuse color.
                  end;
const
  // Flexible vector format (FVF) description.
  // Flexible vector format (FVF) descriptions.
  D3DFVF_OBJECTVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE;
  D3DFVF_MIRRORVERTEX = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_DIFFUSE;

procedure TForm1.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  // Initialize VertexBuffer
  InitVertexBuffer;
  // Turn off D3D lighting, since we are providing our own vertex colors
  SetRenderState(D3DRS_LIGHTING, Ord(false));
  // Enable dithering.
  SetRenderState(D3DRS_DITHERENABLE, 1);
  // Initialize matrices
  InitMatrices;
 end;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
 begin
  OmegaScreen1.ToggleFullScreen;
  InitMatrices;
 end;
 OmegaScreen1.BeginRender;
 OmegaScreen1.ClearScreen(55, 0, 0);
 with OmegaScreen1.Device do
 begin
  // Clear texture
  SetTexture(0, nil);
  FrameMove;
  RenderScene;  // Render the scene
  RenderMirror; // Render the scene in the mirror
  SetRenderState(D3DRS_ALPHABLENDENABLE, cardinal(TRUE));
 end;

 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');

 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 MeshMirror := NIL;
 VertexBuffer := NIL;
end;

procedure TForm1.InitMatrices;
begin
  // Initialize the camera's orientation
  vEyePt    := D3DXVECTOR3(0.0, 2.0, -6.5);
  vLookatPt := D3DXVECTOR3(0.0, 0.0,  0.0);
  vUpVec    := D3DXVECTOR3(0.0, 1.0,  0.0);
  // World.
  D3DXMatrixIdentity(matWorld);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);
  // View.
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);
  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 512/384, 1.0, 100.0);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
end;

procedure TForm1.FrameMove;
var _time : single;
begin
  _time := GetTickCount / 750;
  // Set the teapot's local matrix (rotating about the y-axis)
  D3DXMatrixRotationY(matObj, _time);
  // Move the camera about the z-axis
  vEyePt.x := 4.0 * sin(_time/2.9);
  vEyePt.y := 2.0 * cos(_time/3.7);
  vEyePt.z := -sqrt(40.0 - vEyePt.x*vEyePt.x - vEyePt.y*vEyePt.y);
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);
end;

procedure TForm1.RenderScene;
var _matLocal, _matWorldSaved : TD3DXMatrix;
begin
  // Get a copy of the world matrix.
  OmegaScreen1.Device.GetTransform(D3DTS_WORLD, _matWorldSaved);
  // Build the local matrix.
  D3DXMatrixMultiply(_matLocal, matObj, _matWorldSaved);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, _matLocal);
  // Set the stream source.
  OmegaScreen1.Device.SetStreamSource(0, VertexBuffer, sizeof(TCustomVertex));
  // Set the vertex shader.
  OmegaScreen1.Device.SetVertexShader(D3DFVF_OBJECTVERTEX);
  // Render the vertices.
  OmegaScreen1.Device.DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2);
  // Restore the world matrix
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, _matWorldSaved);
end;

procedure TForm1.RenderMirror;
var _matWorldSaved, _matReflectInMirror : TD3DXMatrix;
    _a, _b, _c, _d : TD3DXVector3;
    _plane : TD3DXPlane;
    _p : pointer;
begin
 with OmegaScreen1.Device do
 begin
  // Save the world matrix so it can be restored
  GetTransform(D3DTS_WORLD, _matWorldSaved);
  // Get the four corners of the mirror.
  // (This should be dynamic rather than hardcoded.)
  _a := D3DXVector3(-1.5, 1.5, 3.0);
  _b := D3DXVector3( 1.5, 1.5, 3.0);
  _c := D3DXVector3(-1.5,-1.5, 3.0);
  _d := D3DXVector3( 1.5,-1.5, 3.0);
  // Construct the reflection matrix
  D3DXPlaneFromPoints(_plane, _a, _b, _c);
  D3DXMatrixReflect(_matReflectInMirror, _plane);
  SetTransform(D3DTS_WORLD, _matReflectInMirror);
  // Reverse the cull mode (since normals will be reflected)
  SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
  // Set the custom clip planes (so geometry is clipped by mirror edges).
  // This is the heart of this sample. The mirror has 4 edges, so there are
  // 4 clip planes, each defined by two mirror vertices and the eye point.
  _p := D3DXPlaneFromPoints(_plane, _b, _a, vEyePt);
  SetClipPlane( 0, _p);
  _p := D3DXPlaneFromPoints( _plane, _d, _b, vEyePt);
  SetClipPlane( 1, _p);
  _p := D3DXPlaneFromPoints( _plane, _c, _d, vEyePt);
  SetClipPlane( 2, _p);
  _p := D3DXPlaneFromPoints( _plane, _a, _c, vEyePt);
  SetClipPlane( 3, _p);
  SetRenderState(D3DRS_CLIPPLANEENABLE,
        D3DCLIPPLANE0 or D3DCLIPPLANE1 or D3DCLIPPLANE2 or D3DCLIPPLANE3 );
  // Render the scene
  RenderScene;
  // Restore the modified render states
  SetTransform(D3DTS_WORLD, _matWorldSaved);
  SetRenderState(D3DRS_CLIPPLANEENABLE, 0);
  SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
  // Finally, render the mirror itself (as an alpha-blended quad)
  SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
  SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  // Set the stream source.
  SetStreamSource(0, MeshMirror, sizeof(TMirrorVertex));
  // Set the vertex shader.
  SetVertexShader(D3DFVF_MIRRORVERTEX);
  // Render the vertices.
  DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
  SetRenderState(D3DRS_ALPHABLENDENABLE, 0);
 end; 
end;

function GetCustomVertex(_x, _y, _z: single;
                         _dcolor: dWord): TCustomVertex;
begin
  Result.X := _x;
  Result.Y := _y;
  Result.Z := _z;
  Result.DColor := _dcolor;
end;

function GetMirrorVertex(_x, _y, _z, _nx, _ny, _nz: single;
                         _dcolor: dWord): TMirrorVertex;
begin
  Result.Position.X := _x;
  Result.Position.Y := _y;
  Result.Position.Z := _z;
  Result.Normal.X := _nx;
  Result.Normal.Y := _ny;
  Result.Normal.Z := _nz;
  Result.DColor := _dcolor;
end;

function TForm1.InitVertexBuffer: boolean;
var _vb1 : array[0..5] of TCustomVertex;
    _vb2 : array[0..3] of TMirrorVertex;
    _ptr : pByte;
begin
  Result := false;
  // Init geometry - object
  _vb1[0] := GetCustomVertex(-1, -1, 0.0, $FFFF0000);
  _vb1[1] := GetCustomVertex( 1, -1, 0.0, $FF00FF00);
  _vb1[2] := GetCustomVertex( 0,  1, 0.0, $FF0000FF);
  _vb1[3] := GetCustomVertex( 0,  1, 0.0, $FFFF00FF);
  _vb1[4] := GetCustomVertex( 1, -1, 0.0, $FF00FFFF);
  _vb1[5] := GetCustomVertex(-1, -1, 0.0, $FFFFFF00);
  // Init geometry - mirror
  _vb2[0] := GetMirrorVertex(-1.5,-1.5, 3.0, 0.0, 0.0,-1.0, $40FFFFFF);
  _vb2[1] := GetMirrorVertex(-1.5, 1.5, 3.0, 0.0, 0.0,-1.0, $40FFFFFF);
  _vb2[2] := GetMirrorVertex( 1.5,-1.5, 3.0, 0.0, 0.0,-1.0, $40FFFFFF);
  _vb2[3] := GetMirrorVertex( 1.5, 1.5, 3.0, 0.0, 0.0,-1.0, $40FFFFFF);
  // Create object vertex buffer.
  if failed(OmegaScreen1.Device.CreateVertexBuffer(SizeOf(_vb1), 0,
                                       D3DFVF_OBJECTVERTEX,
                                       D3DPOOL_MANAGED,
                                       VertexBuffer)) then exit;
  // Create mirror vertex buffer.
  if failed(OmegaScreen1.Device.CreateVertexBuffer(sizeof(_vb2), 0,
                                       D3DFVF_MIRRORVERTEX,
                                       D3DPOOL_MANAGED,
                                       MeshMirror)) then exit;
  // Lock, fill and unlock object vertex buffer.
  if failed(VertexBuffer.Lock(0, 0, _ptr, 0)) then exit;
  Move(_vb1, _ptr^, SizeOf(_vb1));
  VertexBuffer.Unlock;
  // Lock, fill and unlock mirror vertex buffer.
  if failed(MeshMirror.Lock(0, 0, _ptr, 0)) then exit;
  Move(_vb2, _ptr^, SizeOf(_vb2));
  MeshMirror.Unlock;
  // All right
  Result := true;
end;

end.


