unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
  private
    procedure SetMaterials;
    procedure SetupMatrices;
    procedure AnimateLightMap(_timeelapsed: Integer);
    function InitVertexBuffer: boolean;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

var
  VertexBuffer : IDirect3DVertexBuffer8;
  Texture1 : IDirect3DTexture8;
  Texture2 : IDirect3DTexture8;
  canmulti, drawmulti: boolean;
  
  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    Position : TD3DXVector3; // Position
                    U, V     : Single;
                    U2, V2   : single; // U and V for lightmap
                  end;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_TEX1 or D3DFVF_TEX2;

  // Number of faces
  FACENUM = 2;

const
  TEXTFILENAME1 = 'WALL.PNG';
  TEXTFILENAME2 = 'LIGHTMAP.PNG';

procedure TForm1.SetMaterials;
var _mat : TD3DMaterial8;
begin
  // Set Material
  FillChar(_mat, sizeof(_mat), 0 );
  with _mat do
    begin
      Ambient.r := 1.0;
      Ambient.g := 1.0;
      Ambient.b := 1.0;
      Ambient.a := 1.0;
      OmegaScreen1.Device.SetMaterial(_mat);
    end;
end;

procedure TForm1.SetupMatrices;
var _v1, _v2, _v3 : TD3DXVector3;
    _time : dWord;
begin
  // World.
  D3DXMatrixIdentity(matWorld);
  OmegaScreen1.Device.SetTransform(D3DTS_WORLD, matWorld);

  // Rotate the camera
  _time := GetTickCount;
  _v1.x := -cos(_time/3000.0) * 2.0;
  _v1.y := 0.0;
  _v1.z :=  sin(_time/3000.0) * 2.0;
  _v2 := D3DXVector3(0.0, 0.0, 0.0);
  _v3 := D3DXVector3(0.0, 1.0, 0.0);
  // View.
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  OmegaScreen1.Device.SetTransform(D3DTS_VIEW, matView);
  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1, 0.5, 10.0);
  OmegaScreen1.Device.SetTransform(D3DTS_PROJECTION, matProj);
end;

procedure TForm1.AnimateLightMap(_timeelapsed: longint);
var _ptr : PCustomVertex;
    _dec : boolean;
    _c : longint;
begin
  if not failed(VertexBuffer.Lock(0, 0, pByte(_ptr), 0)) then
    begin
      _dec := (_ptr^.U2 > 1.0);
      for _c := 0 to (FACENUM * 3) - 1 do
        begin
          _ptr^.U2 := _ptr^.U2 + 0.001 * _timeelapsed;
          if _dec then _ptr^.U2 := _ptr^.U2 - 1.0;
          inc(_ptr);
        end;
      VertexBuffer.Unlock;
    end;
end;

function GetCustomVertex(_px, _py, _pz, _u, _v: single): TCustomVertex;
begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
     U := _u;
     V := _v;
     U2 := _u;
     V2 := _v;
   end;
end;

function TForm1.InitVertexBuffer: boolean;
var _vb  : array[0..5] of TCustomVertex;
    _ptr : pByte;
begin
  Result := false;
  // Initialize the vertices.
  _vb[ 0] := GetCustomVertex( -1.0, -1.0,  0.0, 0.0, 1.0);
  _vb[ 1] := GetCustomVertex(  1.0,  1.0,  0.0, 1.0, 0.0);
  _vb[ 2] := GetCustomVertex(  1.0, -1.0,  0.0, 1.0, 1.0);

  _vb[ 3] := _vb[0];
  _vb[ 4] := GetCustomVertex( -1.0,  1.0,  0.0, 0.0, 0.0);
  _vb[ 5] := _vb[1];

  // Create a vertex buffer.
  OmegaScreen1.Device.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_MANAGED,
                                       VertexBuffer);
  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer.Unlock;
  // All right
  Result := true;
end;

procedure TForm1.FormCreate(Sender: TObject);
var _filt : dWord;
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 with OmegaScreen1.Device do
 begin
  // Initialize VertexBuffer
  InitVertexBuffer;
  // Set Material
  SetMaterials;
  // Device supports multitexturing ?
  canmulti := omegascreen1.FD3DDEVCAPS8.MaxSimultaneousTextures > 1;
  // Device supports bilinear filter ?
  _filt := (D3DPTFILTERCAPS_MAGFLINEAR or D3DPTFILTERCAPS_MINFLINEAR);
  if (omegascreen1.FD3DDEVCAPS8.TextureFilterCaps and _filt = _filt) then
    begin
      SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
      SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
      if (omegascreen1.FD3DDEVCAPS8.MaxSimultaneousTextures > 1) then
        begin
          SetTextureStageState(1, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
          SetTextureStageState(1, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
        end;
    end;
  // Disable culling.
  SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
  // Set ambiental light.
  SetRenderState(D3DRS_AMBIENT, $FFFFFF);
  // Enable dithering
  SetRenderState(D3DRS_DITHERENABLE, 1);
  // Load textures.
  D3DXCreateTextureFromFileA(omegascreen1.device, TEXTFILENAME1, Texture1);
  D3DXCreateTextureFromFileA(omegascreen1.device, TEXTFILENAME2, Texture2);
 end;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
  OmegaScreen1.ToggleFullScreen;
 OmegaScreen1.ClearScreen(0, 0, 0);
 OmegaScreen1.BeginRender;
 with OmegaScreen1.device do
 begin
  // Set Transform states.
  SetupMatrices;
  // Set the stream source.
  SetStreamSource(0, VertexBuffer, sizeof(TCustomVertex));
  // Set the vertex shader.
  SetVertexShader(D3DFVF_CUSTOMVERTEX);
  AnimateLightmap(10);
  // Draw using multi-pass
  if (canmulti and drawmulti) then
  begin
   // Draw the first textures normally.
   // Use the 1st set of tex coords.
   SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
   SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
   SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
   SetTexture(0, Texture1);
   DrawPrimitive(D3DPT_TRIANGLELIST, 0, FACENUM);
   // Draw the lightmap using blending,
   // with the 2nd set of tex coords
   SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 1);
   SetRenderState(D3DRS_ALPHABLENDENABLE, 1);
   SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ZERO);
   SetRenderState(D3DRS_DESTBLEND, D3DBLEND_SRCCOLOR);
   SetTexture(0, Texture2);
   DrawPrimitive(D3DPT_TRIANGLELIST, 0, FACENUM);
   // Restore state
   SetRenderState(D3DRS_ALPHABLENDENABLE, 0);
  end else
  begin // Draw using multi-texture
   // Texture States for first texture (0); the wall
   SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
   SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
   SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
   SetTexture(0, Texture1);
   // Texture States for second texture (1); the lightmap
   SetTextureStageState(1, D3DTSS_TEXCOORDINDEX, 1);
   SetTextureStageState(1, D3DTSS_COLORARG1, D3DTA_TEXTURE);
   SetTextureStageState(1, D3DTSS_COLORARG2, D3DTA_CURRENT);
   SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_MODULATE);
   SetTexture(1, Texture2);
   // Draw mesh.
   DrawPrimitive(D3DPT_TRIANGLELIST, 0, FACENUM);
   // Restore state
   SetTextureStageState(1, D3DTSS_COLOROP, D3DTOP_DISABLE);
  end;
  SetTextureStageState(0, D3DTSS_TEXCOORDINDEX, 0);
  SetRenderState(D3DRS_ALPHABLENDENABLE, cardinal(TRUE));
  SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
 end;
 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 510, 'Press "space" to toggle fullscreen');
 OmegaBitmapFont1.Print(44, 530, 'Press "M" to toggle multi-texture drawing');
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin                       
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
 if key = ord('M') then drawmulti := not drawmulti;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
 Texture1 := NIL;
 Texture2 := NIL;
 // Free Vertex Buffer
 VertexBuffer := NIL;
end;

end.





