//-----------------------------------------------------------------------------
// File: DotProduct3.cpp
//
// Desc: D3D sample showing how to do bumpmapping using the DotProduct3
//       texture operation.
//
// Copyright (c) 1998-2001 Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, OmegaImageList, OmegaScreen, OmegaInput, OmegaBitmapFont,
  Math, D3DUtil, OmegaTimer, OmegaUtils, D3DX8, {$I UseD3D8.inc};

const
  IDM_USECUSTOMTEXTURE           = 40012;
  IDM_SHOWNORMALMAP              = 40013;
  IDM_USEFILEBASEDTEXTURE        = 40014;
  e                              = 40015;

type
  TCustomVertex = packed record
    v: TD3DXVector3;
    diffuse: DWord;
    specular: DWord;
    tu, tv: Single;
  end;

  PCustomVertexArray = ^TCustomVertexArray;
  TCustomVertexArray = array [0..0] of TCustomVertex;

const
  CUSTOMVERTEX_FVF = (D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_SPECULAR or D3DFVF_TEX1);
  CheckArray: array [False..True] of DWORD = (MF_UNCHECKED, MF_CHECKED);

type
  TForm1 = class(TForm)
    OmegaScreen1: TOmegaScreen;
    OmegaTimer1: TOmegaTimer;
    OmegaBitmapFont1: TOmegaBitmapFont;
    OmegaInput1: TOmegaInput;
    procedure FormCreate(Sender: TObject);
    procedure OmegaTimer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
   private
    m_QuadVertices:             array[0..3] of TCustomVertex;
    m_pCustomNormalMap:         IDirect3DTexture8;
    m_pFileBasedNormalMap:      IDirect3DTexture8;
    m_vLight:                   TD3DXVector3;
    m_bUseFileBasedTexture:     Boolean;
    m_bShowNormalMap:           Boolean;
    function CreateFileBasedNormalMap: HResult;
    function CreateCustomNormalMap: HResult;
    procedure ResetData;
  end;

var Form1: TForm1;

implementation
{$R *.DFM}

//-----------------------------------------------------------------------------
// Name: VectortoRGBA()
// Desc: Turns a normalized vector into RGBA form. Used to encode vectors into
//       a height map.
//-----------------------------------------------------------------------------
function VectortoRGBA(const v: TD3DXVector3; fHeight: Single): DWord;
var
  r, g, b, a: DWord;
begin
  r := Round(127.0 * v.x + 128.0);
  g := Round(127.0 * v.y + 128.0);
  b := Round(127.0 * v.z + 128.0);
  a := Round(255.0 * fHeight);
  Result:= ( (a shl 24) + (r shl 16) + (g shl 8) + (b shl 0));
end;

//-----------------------------------------------------------------------------
// Name: InitVertex()
// Desc: Initializes a vertex
//-----------------------------------------------------------------------------
procedure InitVertex(var vtx: TCustomVertex; x, y, z, tu, tv: Single);
var
  v: TD3DXVector3;
begin
  v:= D3DXVector3(1,1,1);
  D3DXVec3Normalize(v, v);
  vtx.v        := D3DXVector3( x, y, z);
  vtx.diffuse  := VectortoRGBA(v, 1.0);
  vtx.specular := $40400000;
  vtx.tu       := tu;
  vtx.tv       := tv;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
 self.ClientWidth := 800;
 self.ClientHeight := 600;
 OmegaScreen1.Width := self.ClientWidth;
 OmegaScreen1.Height := self.ClientHeight;
 OmegaScreen1.Init;
 InitVertex(m_QuadVertices[0],-1.0,-1.0,-1.0, 0.0, 0.0);
 InitVertex(m_QuadVertices[1], 1.0,-1.0,-1.0, 1.0, 0.0);
 InitVertex(m_QuadVertices[2],-1.0, 1.0,-1.0, 0.0, 1.0);
 InitVertex(m_QuadVertices[3], 1.0, 1.0,-1.0, 1.0, 1.0);
 m_vLight := D3DXVector3(0.0, 0.0, 1.0);
 CreateFileBasedNormalMap;
 CreateCustomNormalMap;
 ResetData;
 OmegaTimer1.Enabled := True;
end;

procedure TForm1.ResetData;
var
  vEyePt, vLookatPt, vUpVec: TD3DXVector3;
  matWorld, matView, matProj: TD3DXMatrix;
  fAspect: Single;
begin
  // Set the transform matrices
  vEyePt    := D3DXVector3(0.0, 0.0, 2.0);
  vLookatPt := D3DXVector3(0.0, 0.0, 0.0);
  vUpVec    := D3DXVector3(0.0, 1.0, 0.0);

  D3DXMatrixIdentity(matWorld);
  D3DXMatrixLookAtLH(matView, vEyePt, vLookatPt, vUpVec);
  fAspect:= ClientWidth / ClientHeight;
  D3DXMatrixPerspectiveFovLH(matProj, D3DX_PI/4, fAspect, 1.0, 500.0);
  with OmegaScreen1.Device do
  begin
   SetTransform(D3DTS_WORLD,      matWorld);
   SetTransform(D3DTS_VIEW,       matView);
   SetTransform(D3DTS_PROJECTION, matProj);
   // Set misc render states
   SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
   SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
   SetRenderState(D3DRS_LIGHTING, iFALSE);
  end; 
end;

procedure TForm1.OmegaTimer1Timer(Sender: TObject);
var
  pt: TPoint;
  dwFactor: DWord;
begin
 if not OmegaScreen1.CanDraw then exit;
 OmegaInput1.Update;
 if oisbutton1 in OmegaInput1.keyboard.statesclicked then
 begin
  OmegaScreen1.ToggleFullScreen;
  ResetData;
 end;
 OmegaScreen1.ClearScreen(0, 0, 0);
 OmegaScreen1.BeginRender;
 // Compute the light vector from the cursor position
 if GetFocus <> 0 then
 begin
  GetCursorPos(pt);
  pt := ScreenToClient(pt);
  m_vLight.x := -(((2.0 * pt.x ) / ClientWidth ) - 1);
  m_vLight.y := -(((2.0 * pt.y ) / ClientHeight ) - 1);
  m_vLight.z := 0.0;
  if (D3DXVec3Length(m_vLight) > 1.0) then
   D3DXVec3Normalize(m_vLight, m_vLight)
  else
   m_vLight.z := Sqrt( 1.0 - m_vLight.x*m_vLight.x
                           - m_vLight.y*m_vLight.y);
 end;
 // Store the light vector, so it can be referenced in D3DTA_TFACTOR
 dwFactor := VectortoRGBA(m_vLight, 0.0);
 with OmegaScreen1.Device do
 begin
  SetRenderState(D3DRS_TEXTUREFACTOR, dwFactor);
  // Modulate the texture (the normal map) with the light vector (stored
  // above in the texture factor)
  SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_DOTPRODUCT3);
  SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_TFACTOR);
  // If user wants to see the normal map, override the above renderstates and
  // simply show the texture
  if m_bShowNormalMap then
   SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
  // Select which normal map to use
  if m_bUseFileBasedTexture then
   SetTexture(0, m_pFileBasedNormalMap)
  else SetTexture(0, m_pCustomNormalMap);
  // Draw the bumpmapped quad
  SetVertexShader(CUSTOMVERTEX_FVF);
  DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, m_QuadVertices, SizeOf(TCustomVertex));
 end;
 OmegaScreen1.ResetRenderStates;
 OmegaScreen1.SetBlendMode(bmNormal);
 OmegaBitmapFont1.Print(44, 450, 'Press "space" to toggle fullscreen');
 OmegaBitmapFont1.Print(44, 470, 'Press "T" to toggle texture');
 OmegaBitmapFont1.Print(44, 490, 'Press "N" to toggle normal map');
 OmegaBitmapFont1.Print(44, 510, 'Move mouse to change lighting');
 OmegaScreen1.EndRender;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
 OmegaTimer1.Enabled := False;
end;

procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
 if key = 27 then close;
 if key = ord('T') then
  m_bUseFileBasedTexture := not m_bUseFileBasedTexture;
 if key = ord('N') then
  m_bShowNormalMap := not m_bShowNormalMap
end;

function TForm1.CreateFileBasedNormalMap: HResult;
var
  d3dlr: TD3DLockedRect;
  d3dsd: TD3DSurfaceDesc;
  pPixel: PDWordArray;

  j, i: Integer;
  color00, color10, color01: DWord;
  fHeight00, fHeight10, fHeight01: Single;

  vPoint00: TD3DXVector3;
  vPoint10: TD3DXVector3;
  vPoint01: TD3DXVector3;
  v10, v01: TD3DXVector3;

  vNormal: TD3DXVector3;
begin
  // Load the texture from a file
  D3DUtil_CreateTexture(OmegaScreen1.Device, 'EarthBump.bmp',
   m_pFileBasedNormalMap, D3DFMT_A8R8G8B8);
  // Lock the texture
  m_pFileBasedNormalMap.GetLevelDesc(0, d3dsd);
  m_pFileBasedNormalMap.LockRect(0, d3dlr, nil, 0);
  pPixel:= PDWordArray(d3dlr.pBits);

  // For each pixel, generate a vector normal that represents the change
  // in thea height field at that pixel
  for j:= 0 to (d3dsd.Height - 1) do
  begin
    for i:= 0 to (d3dsd.Width - 1) do
    begin
      color00 := pPixel[0];
      color10 := pPixel[1];
      color01 := pPixel[d3dlr.Pitch div SizeOf(DWord)];

      fHeight00 := ((color00 and $00ff0000) shr 16) / 255.0;
      fHeight10 := ((color10 and $00ff0000) shr 16) / 255.0;
      fHeight01 := ((color01 and $00ff0000) shr 16) / 255.0;

      vPoint00:= D3DXVector3(i+0.0, j+0.0, fHeight00);
      vPoint10:= D3DXVector3(i+1.0, j+0.0, fHeight10);
      vPoint01:= D3DXVector3(i+0.0, j+1.0, fHeight01);
      D3DXVec3Subtract(v10, vPoint10, vPoint00);
      D3DXVec3Subtract(v01, vPoint01, vPoint00);

      D3DXVec3Cross(vNormal, v10, v01);
      D3DXVec3Normalize(vNormal, vNormal);

      // Store the normal as an RGBA value in the normal map
      pPixel[0]:= VectortoRGBA(vNormal, fHeight00);
      Inc(PDWord(pPixel))
    end;
  end;

  // Unlock the texture and return successful
  m_pFileBasedNormalMap.UnlockRect(0);

  Result := S_OK;
end;

function TForm1.CreateCustomNormalMap: HResult;
var
  dwWidth, dwHeight: DWord;
  d3dlr: TD3DLockedRect;
  pPixel: PDWord;
  i, j: Integer;
  xp, yp, x, y, z: Single;
  vVector: TD3DXVector3;
begin
  dwWidth  := 512;
  dwHeight := 512;
  // Create a 32-bit texture for the custom normal map
  Result:= OmegaScreen1.Device.CreateTexture(dwWidth, dwHeight, 1, 0 (* Usage *),
             D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, m_pCustomNormalMap);
  if FAILED(Result) then Exit;

  // Lock the texture to fill it with our custom image
  if FAILED(m_pCustomNormalMap.LockRect(0, d3dlr, nil, 0)) then
  begin
    Result := E_FAIL;
    Exit;
  end;
  pPixel:= PDWord(d3dlr.pBits);

  // Fill each pixel
  for j:= 0 to (dwHeight - 1) do
  begin
    for i:= 0 to (dwWidth - 1) do
    begin
      xp := ((5.0*i) / (dwWidth  - 1));
      yp := ((5.0*j) / (dwHeight - 1));

      x  := 2*(xp-floor(xp))-1;
      y  := 2*(yp-floor(yp))-1;
      // z  := Sqrt(1.0 - x*x - y*y);
        // - some minor error in C++ code param to Sqrt can be less then ZERO)
        // -> moved this line below
        // or we could used <fclex> to clear interrupts flags :BAA

      // Make image of raised circle. Outside of circle is gray
      if ((x*x + y*y) <= 1.0) then
      begin
        z  := Sqrt(1.0 - x*x - y*y); // moved line (look above) :BAA
        vVector:= D3DXVector3( x, y, z);
        pPixel^:= VectortoRGBA(vVector, 1.0);
      end else
        pPixel^:= $80808080;

      Inc(pPixel);
    end;
  end;

  // Unlock the map and return successful
  m_pCustomNormalMap.UnlockRect(0);

  Result := S_OK;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  m_pFileBasedNormalMap:= nil;
  m_pCustomNormalMap:= nil;
end;

end.





