unit OmegaAnimationList;

interface
uses
  Classes, SysUtils, OmegaImageList, OmegaSprite, OmegaUtils;

type
  TOmegaAnimation = class;
  TOmegaAnimationList = class;
  TAdvancedAnimationSprite = class;

  TOmegaAnimationCommand = (fcPlay, fcStop, fcDie, fcGotoFrame, fcGotoAnim, fcLoop);
  TOmegaAnimationAction = (aaImage, aaPosition, aaColor, aaAlpha, aaRotate, aaScale);
  TOmegaAnimationActions = set of TOmegaAnimationAction;
  TOmegaAnimationFrame = class(TCollectionItem)
  private
    FName: String;
    FCommand: TOmegaAnimationCommand;
    FDuration: Single;
    FActions: TOmegaAnimationActions;
    //Generic
    FNewIndex: Integer;
    //Loop
    FLoopCount: Integer;
    //Image
    FImageListItem: Integer;
    //Position
    FFrameAnimX, FFrameAnimY: Single;
    //Color
    FFrameRed, FFrameGreen, FFrameBlue: Integer;
    //Alpha
    FFrameAlpha: Integer;
    //Rotation
    FFrameRotation: Single;
    //Scale
    FFrameScaleX, FFrameScaleY: Single;
  protected
    function GetDisplayName : String; override;
  public
    constructor Create(Collection: TCollection); override;
    procedure Assign(Source: TPersistent); override;
  published
    property Name: String read FName write FName;
    property Command: TOmegaAnimationCommand read FCommand write FCommand;
    property Duration: Single read FDuration write FDuration;
    property Actions: TOmegaAnimationActions read FActions write FActions;
    //Generic
    property NewIndex: Integer read FNewIndex write FNewIndex;
    //Loop
    property LoopCount: Integer read FLoopCount write FLoopCount;
    //Image
    property ImageListItem: Integer read FImageListItem write FImageListItem;
    //Position
    property FrameAnimX: Single read FFrameAnimX write FFrameAnimX;
    property FrameAnimY: Single read FFrameAnimY write FFrameAnimY;
    //Color
    property FrameRed: Integer read FFrameRed write FFrameRed;
    property FrameGreen: Integer read FFrameGreen write FFrameGreen;
    property FrameBlue: Integer read FFrameBlue write FFrameBlue;
    //Alpha
    property FrameAlpha: Integer read FFrameAlpha write FFrameAlpha;
    //Rotation
    property FrameRotation: Single read FFrameRotation write FFrameRotation;
    //Scale
    property FrameScaleX: Single read FFrameScaleX write FFrameScaleX;
    property FrameScaleY: Single read FFrameScaleY write FFrameScaleY;
  end;

  TOmegaAnimationFrameCollection = class(TOwnedCollection)
  private
    FAnimation: TOmegaAnimation;
  protected
    function GetItem(Index: Integer): TOmegaAnimationFrame;
    procedure SetItem(Index: Integer; Value: TOmegaAnimationFrame);
  public
    constructor Create(AOwner: TOmegaAnimation);
    property Animation: TOmegaAnimation read FAnimation;
    function Add: TOmegaAnimationFrame;
    function AddPlay(const ADuration: Single; const AName: String; const AActions: TOmegaAnimationActions; const TileIndex: Integer = 0; const ImageListItemIndex: Integer = 0; const AAnimX: Single = 0; const AAnimY: Single = 0; const AAlpha: Integer = 0; const ARotation: Single = 0; const AScaleX: Single = 0; const AScaleY: Single = 0; const ARed: Integer = 0; const AGreen: Integer = 0; const ABlue: Integer = 0): TOmegaAnimationFrame; overload;
    function AddPlay(const ADuration: Single; const AName: String; const AActions: TOmegaAnimationActions; const TileIndex: Integer = 0; const ImageListItemName: String = '';  const AAnimX: Single = 0; const AAnimY: Single = 0; const AAlpha: Integer = 0; const ARotation: Single = 0; const AScaleX: Single = 0; const AScaleY: Single = 0; const ARed: Integer = 0; const AGreen: Integer = 0; const ABlue: Integer = 0): TOmegaAnimationFrame; overload;
    function AddStop: TOmegaAnimationFrame;
    function AddDie: TOmegaAnimationFrame;
    function AddGotoFrame(const FrameIndex: Integer): TOmegaAnimationFrame; overload;
    function AddGotoFrame(const FrameName: String): TOmegaAnimationFrame; overload;
    function AddGotoAnim(const AnimIndex: Integer): TOmegaAnimationFrame; overload;
    function AddGotoAnim(const AnimName: String): TOmegaAnimationFrame; overload;
    function AddLoop(const FrameIndex, Loops: Integer): TOmegaAnimationFrame; overload;
    function AddLoop(const FrameName: String; const Loops: Integer): TOmegaAnimationFrame; overload;
    function Insert(Index: Integer): TOmegaAnimationFrame;
    procedure LoadFromFile(const Filename: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const Filename: string);
    procedure SaveToStream(Stream: TStream);
    property Items[Index: Integer]: TOmegaAnimationFrame read GetItem write SetItem;
    function Find(AName: string): TOmegaAnimationFrame;
  end;

  TOmegaAnimation = class(TCollectionItem)
  private
    FName: String;
    FFrames: TOmegaAnimationFrameCollection;
    procedure SetFrames(const Value: TOmegaAnimationFrameCollection);
  protected
    function GetDisplayName : String; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure FindNext(var AFrame: TOmegaAnimationFrame; var AAnimation: TOmegaAnimation; var ALoopCount: Integer; var AReturnAnimation: TOmegaAnimation);
  published
    property Name: String read FName write FName;
    property Frames: TOmegaAnimationFrameCollection read FFrames write SetFrames;
  end;

  TOmegaAnimationCollection = class(TOwnedCollection)
  private
    FAnimationList: TOmegaAnimationList;
  protected
    function GetItem(Index: Integer): TOmegaAnimation;
    procedure SetItem(Index: Integer; Value: TOmegaAnimation);
  public
    constructor Create(AOwner: TOmegaAnimationList);
    property AnimationList: TOmegaAnimationList read FAnimationList;
    function Add: TOmegaAnimation; overload;
    function Add(const AName: String): TOmegaAnimation; overload;
    function Insert(Index: Integer): TOmegaAnimation;
    function Find(AName: string): TOmegaAnimation;
    procedure LoadFromFile(const Filename: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const Filename: string);
    procedure SaveToStream(Stream: TStream);
    property Items[Index: Integer]: TOmegaAnimation read GetItem write SetItem;
  end;

  TOmegaAnimationList = class(TComponent)
  private
    FOmegaImageList: TOmegaImageList;
    FAnimations: TOmegaAnimationCollection;
    procedure SetAnimations(const Value: TOmegaAnimationCollection);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Animations: TOmegaAnimationCollection read FAnimations write SetAnimations;
  published
    property OmegaImageList: TOmegaImageList read FOmegaImageList write FOmegaImageList;
  end;

  TOmegaAnimationPlayer = class(TPersistent)
  private
    FAnimation, FNextAnimation, FReturnAnimation: TOmegaAnimation;
    FFrame, FNextFrame: TOmegaAnimationFrame;
    FAnimationStopped: Boolean;
    FFrameTime: Single;
    FLoopCount: Integer;
    procedure SetAnimation(const Value: TOmegaAnimation);
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;
    procedure Progress(MoveCount: Single; const Sprite: TAdvancedAnimationSprite);
    procedure ResetAnimation;
    procedure GosubAnimation(const Value: TOmegaAnimation);
    property Animation: TOmegaAnimation read FAnimation write SetAnimation;
  end;

  TAdvancedAnimationSprite = class(TSprite)
  private
    FPlayerCount: Integer;
    FPlayers: array of TOmegaAnimationPlayer;
    FBaseAlpha: Integer;
    FBaseX, FBaseY, FBaseRotation, FBaseScaleX, FBaseScaleY: Single;
    FAnimationAlpha: Integer;
    FAnimationX, FAnimationY, FAnimationRotation, FAnimationScaleX, FAnimationScaleY: Single;

    procedure SetBaseX(const Value: Single);
    procedure SetBaseY(const Value: Single);
    procedure SetBaseRotation(const Value: Single);
    procedure SetBaseAlpha(const Value: Integer);
    procedure SetBaseScaleX(const Value: Single);
    procedure SetBaseScaleY(const Value: Single);

    procedure SetAnimationX(const Value: Single);
    procedure SetAnimationY(const Value: Single);
    procedure SetAnimationRotation(const Value: Single);
    procedure SetAnimationAlpha(const Value: Integer);
    procedure SetAnimationScaleX(const Value: Single);
    procedure SetAnimationScaleY(const Value: Single);
    function GetPlayer(Index: Integer): TOmegaAnimationPlayer;
    procedure SetPlayer(Index: Integer;
      const Value: TOmegaAnimationPlayer);
    procedure SetPlayerCount(const Value: Integer);
  protected
  public
    constructor Create(const AParent: TOmegaSprite); override;
    destructor Destroy; override;
    procedure Assign(const Value: TSprite); override;
    procedure Move(const MoveCount: single); override;

    property PlayerCount: Integer read FPlayerCount write SetPlayerCount;
    property Players[Index: Integer]: TOmegaAnimationPlayer read GetPlayer write SetPlayer;

    property BaseX: Single read FBaseX write SetBaseX;
    property BaseY: Single read FBaseY write SetBaseY;
    property BaseRotation: Single read FBaseRotation write SetBaseRotation;
    property BaseAlpha: Integer read FBaseAlpha write SetBaseAlpha;
    property BaseScaleX: Single read FBaseScaleX write SetBaseScaleX;
    property BaseScaleY: Single read FBaseScaleY write SetBaseScaleY;

    property AnimationX: Single read FAnimationX write SetAnimationX;
    property AnimationY: Single read FAnimationY write SetAnimationY;
    property AnimationAlpha: Integer read FAnimationAlpha write SetAnimationAlpha;
    property AnimationRotation: Single read FAnimationRotation write SetAnimationRotation;
    property AnimationScaleX: Single read FAnimationScaleX write SetAnimationScaleX;
    property AnimationScaleY: Single read FAnimationScaleY write SetAnimationScaleY;
  end;

implementation

{ TOmegaAnimationFrame }

constructor TOmegaAnimationFrame.Create(Collection: TCollection);
begin
  FActions := [];
  inherited;
end;

procedure TOmegaAnimationFrame.Assign(Source: TPersistent);
begin
  if Source is TOmegaAnimationFrame then begin
    Name := TOmegaAnimationFrame(Source).Name;
    Command := TOmegaAnimationFrame(Source).Command;
    ImageListItem := TOmegaAnimationFrame(Source).ImageListItem;
    NewIndex := TOmegaAnimationFrame(Source).NewIndex;
    Duration := TOmegaAnimationFrame(Source).Duration;
  end else
    inherited;
end;

function Interpolate(a,b,s:single):integer;
begin
 result:=round((1-s)*a+s*b);
end;

function TOmegaAnimationFrame.GetDisplayName: String;
begin
  Result := 'Frame';
end;

{ TOmegaAnimationFrameCollection }

type //streaming support
  TOmegaAnimationFrameCollectionComponent = class(TComponent)
  private
    FList: TOmegaAnimationFrameCollection;
  published
    property List: TOmegaAnimationFrameCollection read FList write FList;
  end;

constructor TOmegaAnimationFrameCollection.Create(AOwner: TOmegaAnimation);
begin
  inherited Create(AOwner, TOmegaAnimationFrame);
  FAnimation := AOwner;
end;

function TOmegaAnimationFrameCollection.GetItem(Index: Integer): TOmegaAnimationFrame;
begin
  Result := TOmegaAnimationFrame(inherited GetItem(Index));
end;

procedure TOmegaAnimationFrameCollection.SetItem(Index: Integer; Value: TOmegaAnimationFrame);
begin
  inherited SetItem(Index, Value);
end;

function TOmegaAnimationFrameCollection.Add: TOmegaAnimationFrame;
begin
  Result := TOmegaAnimationFrame(inherited Add);
end;

function TOmegaAnimationFrameCollection.Insert(Index: Integer): TOmegaAnimationFrame;
begin
  Result := TOmegaAnimationFrame(inherited Insert(Index));
end;

function TOmegaAnimationFrameCollection.Find(AName: string): TOmegaAnimationFrame;
var
  i: Integer;
begin
  Result := nil;
  for i := Count-1 downto 0 do begin
    if AName = Items[i].FName then begin
      Result := Items[i];
      Exit;
    end;
  end;
end;

procedure TOmegaAnimationFrameCollection.LoadFromFile(const Filename: string); 
var  
  Stream: TFileStream; 
begin 
  Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite); 
  try 
    LoadFromStream(Stream); 
  finally 
    Stream.Free; 
  end; 
end; 
 
procedure TOmegaAnimationFrameCollection.LoadFromStream(Stream: TStream); 
var 
  Component: TOmegaAnimationFrameCollectionComponent; 
begin 
  Clear; 
  Component := TOmegaAnimationFrameCollectionComponent.Create(nil); 
  try
    Component.FList := self; 
    Stream.ReadComponentRes(Component); 
  finally 
    Component.Free; 
  end; 
end;

procedure TOmegaAnimationFrameCollection.SaveToFile(const Filename: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(Filename, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TOmegaAnimationFrameCollection.SaveToStream(Stream: TStream);
var
  Component: TOmegaAnimationFrameCollectionComponent;
begin
  Component := TOmegaAnimationFrameCollectionComponent.Create(nil);
  try
    Component.FList := self;
    Stream.WriteComponentRes('TOmegaAnimationFrameCollection', Component);
  finally
    Component.Free;
  end;
end;

function TOmegaAnimationFrameCollection.AddPlay(const ADuration: Single; const AName: String; const AActions: TOmegaAnimationActions; const TileIndex: Integer = 0; const ImageListItemIndex: Integer = 0; const AAnimX: Single = 0; const AAnimY: Single = 0; const AAlpha: Integer = 0; const ARotation: Single = 0; const AScaleX: Single = 0; const AScaleY: Single = 0; const ARed: Integer = 0; const AGreen: Integer = 0; const ABlue: Integer = 0): TOmegaAnimationFrame;
begin
  Result := Self.Add;
  with Result do begin
    FName := AName;
    FCommand := fcPlay;
    FDuration := ADuration;
    FActions := AActions;

    if aaImage in FActions then begin
      FImageListItem := ImageListItemIndex;
      FNewIndex := TileIndex;
    end;
    if aaPosition in FActions then begin
      FFrameAnimX := AAnimX;
      FFrameAnimY := AAnimY;
    end;
    if aaColor in FActions then begin
      FFrameRed := ARed;
      FFrameGreen := AGreen;
      FFrameBlue := ABlue;
    end;
    if aaAlpha in FActions then
      FFrameAlpha := AAlpha;
    if aaRotate in FActions then
      FFrameRotation := ARotation;
    if aaScale in FActions then begin
      FFrameScaleX := AScaleX;
      FFrameScaleY := AScaleY;
    end;
  end;
end;

function TOmegaAnimationFrameCollection.AddPlay(const ADuration: Single; const AName: String; const AActions: TOmegaAnimationActions; const TileIndex: Integer = 0; const ImageListItemName: String = '';  const AAnimX: Single = 0; const AAnimY: Single = 0; const AAlpha: Integer = 0; const ARotation: Single = 0; const AScaleX: Single = 0; const AScaleY: Single = 0; const ARed: Integer = 0; const AGreen: Integer = 0; const ABlue: Integer = 0): TOmegaAnimationFrame;
var
  i: Integer;
begin
  if aaImage in AActions then
    i := TOmegaAnimationCollection(FAnimation.GetOwner).FAnimationList.FOmegaImageList.ImageList.Find(ImageListItemName).Index
  else
    i := -1;
  Result := AddPlay(ADuration, AName, AActions, TileIndex, i, AAnimX, AAnimY, AAlpha, ARotation, AScaleX, AScaleY, ARed, AGreen, ABlue);
end;

function TOmegaAnimationFrameCollection.AddStop: TOmegaAnimationFrame;
begin
  Result := Add;
  Result.FCommand := fcStop;
end;

function TOmegaAnimationFrameCollection.AddDie: TOmegaAnimationFrame;
begin
  Result := Add;
  Result.FCommand := fcDie;
end;

function TOmegaAnimationFrameCollection.AddGotoFrame(const FrameIndex: Integer): TOmegaAnimationFrame;
begin
  Result := Add;
  with Result do begin
    FCommand := fcGotoFrame;
    FNewIndex := FrameIndex;
  end;
end;

function TOmegaAnimationFrameCollection.AddGotoFrame(const FrameName: String): TOmegaAnimationFrame;
var
  f: TOmegaAnimationFrame;
begin
  f := Find(FrameName);
  if f = nil then
    raise Exception.Create('AddGotoFrame failed: Could not find frame ' + FrameName);
  Result := AddGotoFrame(f.Index);
end;

function TOmegaAnimationFrameCollection.AddGotoAnim(const AnimIndex: Integer): TOmegaAnimationFrame;
begin
  Result := Add;
  with Result do begin
    FCommand := fcGotoAnim;
    FNewIndex := AnimIndex;
  end;
end;

function TOmegaAnimationFrameCollection.AddGotoAnim(const AnimName: String): TOmegaAnimationFrame;
begin
  Result := AddGotoAnim(
    TOmegaAnimationCollection(FAnimation.GetOwner).Find(AnimName).Index
  );
end;

function TOmegaAnimationFrameCollection.AddLoop(const FrameIndex, Loops: Integer): TOmegaAnimationFrame;
begin
  Result := Add;
  with Result do begin
    FCommand := fcLoop;
    FNewIndex := FrameIndex;
    FLoopCount := Loops;
  end;
end;

function TOmegaAnimationFrameCollection.AddLoop(const FrameName: String; const Loops: Integer): TOmegaAnimationFrame;
begin
  Result := AddLoop(Find(FrameName).Index, Loops);
end;

{ TOmegaAnimation }

constructor TOmegaAnimation.Create(Collection: TCollection);
begin
  inherited;
  FFrames := TOmegaAnimationFrameCollection.Create(Self);
end;

destructor TOmegaAnimation.Destroy;
begin
  if FFrames <> nil then FFrames.Free;
  inherited;
end;

procedure TOmegaAnimation.Assign(Source: TPersistent);
begin
  if Source is TOmegaAnimation then begin
    Frames := TOmegaAnimation(Source).Frames;
  end else
    inherited;
end;

function TOmegaAnimation.GetDisplayName: String;
begin
  Result := 'Animation';
end;

procedure TOmegaAnimation.SetFrames(const Value: TOmegaAnimationFrameCollection);
begin
  FFrames.Assign(Value);
end;

procedure TOmegaAnimation.FindNext(var AFrame: TOmegaAnimationFrame; var AAnimation: TOmegaAnimation; var ALoopCount: Integer; var AReturnAnimation: TOmegaAnimation);
var
  i: Integer;
  b: Boolean;
begin
  if AFrame = nil then
    i := -1
  else
    i := AFrame.Index;
  b := False;
  repeat
    Inc(i);
    if i = FFrames.Count then i := 0;
    AFrame := FFrames.Items[i];
    case AFrame.FCommand of
      fcPlay: b := True;
      fcStop: begin
        if AReturnAnimation <> nil then begin
          AAnimation := AReturnAnimation;
          AFrame := nil;
          AAnimation.FindNext(AFrame, AAnimation, ALoopCount, AReturnAnimation);
        end;
        b := True;
      end;
      fcDie:
        b := True;
      fcGotoFrame:
        i := AFrame.FNewIndex - 1;
      fcGotoAnim: begin
        AAnimation := TOmegaAnimation(Collection.Items[AFrame.FNewIndex]);
        AFrame := nil;
        AAnimation.FindNext(AFrame, AAnimation, ALoopCount, AReturnAnimation);
        b := True;
      end;
      fcLoop: begin
        if (ALoopCount = 0) and (AFrame.FLoopCount > 1) then begin
          ALoopCount := AFrame.FLoopCount - 1;
          i := AFrame.FNewIndex - 1;
        end else begin
          if ALoopCount > 1 then
            i := AFrame.FNewIndex - 1;
          Dec(ALoopCount);
        end;
      end;

    end;
  until b;
end;

{ TOmegaAnimationCollection }

type //streaming support
  TOmegaAnimationCollectionComponent = class(TComponent)
  private
    FList: TOmegaAnimationCollection;
  published
    property List: TOmegaAnimationCollection read FList write FList;
  end;

constructor TOmegaAnimationCollection.Create(AOwner: TOmegaAnimationList);
begin
  inherited Create(AOwner, TOmegaAnimation);
  FAnimationList := AOwner;
end;

function TOmegaAnimationCollection.GetItem(Index: Integer): TOmegaAnimation;
begin
  Result := TOmegaAnimation(inherited GetItem(Index));
end;

procedure TOmegaAnimationCollection.SetItem(Index: Integer; Value: TOmegaAnimation);
begin
  inherited SetItem(Index, Value);
end;

function TOmegaAnimationCollection.Add: TOmegaAnimation;
begin
  Result := TOmegaAnimation(inherited Add);
end;

function TOmegaAnimationCollection.Add(const AName: String): TOmegaAnimation;
begin
  Result := TOmegaAnimation(inherited Add);
  Result.FName := AName;
end;

function TOmegaAnimationCollection.Insert(Index: Integer): TOmegaAnimation;
begin
  Result := TOmegaAnimation(inherited Insert(Index));
end;

function TOmegaAnimationCollection.Find(AName: string): TOmegaAnimation;
var
  i: Integer;
begin
  Result := nil;
  for i := Count-1 downto 0 do begin
    if AName = Items[i].FName then begin
      Result := Items[i];
      Exit;
    end;
  end;
end;

procedure TOmegaAnimationCollection.LoadFromFile(const Filename: string); 
var  
  Stream: TFileStream; 
begin 
  Stream := TFileStream.Create(Filename, fmOpenRead or fmShareDenyWrite); 
  try 
    LoadFromStream(Stream); 
  finally 
    Stream.Free; 
  end; 
end; 
 
procedure TOmegaAnimationCollection.LoadFromStream(Stream: TStream); 
var
  Component: TOmegaAnimationCollectionComponent; 
begin 
  Clear;
  Component := TOmegaAnimationCollectionComponent.Create(nil); 
  try 
    Component.FList := self; 
    Stream.ReadComponentRes(Component); 
  finally 
    Component.Free; 
  end; 
end; 
 
procedure TOmegaAnimationCollection.SaveToFile(const Filename: string); 
var 
  Stream: TFileStream; 
begin 
  Stream := TFileStream.Create(Filename, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TOmegaAnimationCollection.SaveToStream(Stream: TStream);
var
  Component: TOmegaAnimationCollectionComponent;
begin
  Component := TOmegaAnimationCollectionComponent.Create(nil);
  try
    Component.FList := self;
    Stream.WriteComponentRes('TOmegaAnimationCollection', Component);
  finally 
    Component.Free; 
  end;
end;

{ TOmegaAnimationList }

constructor TOmegaAnimationList.Create(AOwner: TComponent);
begin
  inherited;
  FAnimations := TOmegaAnimationCollection.Create(Self);
end;

destructor TOmegaAnimationList.Destroy;
begin
  if FAnimations <> nil then FAnimations.Free;
  inherited;
end;

procedure TOmegaAnimationList.SetAnimations(const Value: TOmegaAnimationCollection);
begin
  FAnimations.Assign(Value);
end;

{ TOmegaAnimationPlayer }

constructor TOmegaAnimationPlayer.Create;
begin
  FNextFrame := nil;
  FFrame := nil;
  FReturnAnimation := nil;
  FAnimationStopped := True;
end;

procedure TOmegaAnimationPlayer.Assign(Source: TPersistent);
begin
  if Source is TOmegaAnimationPlayer then begin
    Animation := TOmegaAnimationPlayer(Source).Animation;
  end else
    inherited;
end;

procedure TOmegaAnimationPlayer.Progress(MoveCount: Single; const Sprite: TAdvancedAnimationSprite);
var
  bDone: Boolean;
  part: Single;
begin
  if FAnimationStopped or (FAnimation = nil) then Exit;

  //Setup initial situation
  if FFrame = nil then begin
    FAnimation.FindNext(FFrame, FAnimation, FLoopCount, FReturnAnimation);
    FNextFrame := FFrame;
    FNextAnimation := FAnimation;
    FAnimation.FindNext(FNextFrame, FNextAnimation, FLoopCount, FReturnAnimation);
  end;

  FFrameTime := FFrameTime + MoveCount;
  bDone := False;
  repeat

    case FFrame.FCommand of
      fcPlay: begin
        if FFrameTime < FFrame.FDuration then begin
          //Interpolate a frame
          part := FFrameTime / FFrame.FDuration;
          if aaImage in FFrame.FActions then begin
            Sprite.Image := TOmegaAnimationCollection(TOmegaAnimationFrameCollection(FFrame.Collection).FAnimation.Collection).FAnimationList.FOmegaImageList.ImageList.Items[FFrame.FImageListItem];
            Sprite.ImageIndex := FFrame.FNewIndex;
          end;
          if (aaPosition in FFrame.FActions) and (aaPosition in FNextFrame.FActions) then begin
            Sprite.AnimationX := Interpolate(FFrame.FFrameAnimX, FNextFrame.FFrameAnimX, part);
            Sprite.AnimationY := Interpolate(FFrame.FFrameAnimY, FNextFrame.FFrameAnimY, part);
          end;
          if (aaColor in FFrame.FActions) and (aaColor in FNextFrame.FActions) then begin
            Sprite.Red   := Interpolate(FFrame.FFrameRed,   FNextFrame.FFrameRed, part);
            Sprite.Green := Interpolate(FFrame.FFrameGreen, FNextFrame.FFrameGreen, part);
            Sprite.Blue  := Interpolate(FFrame.FFrameBlue,  FNextFrame.FFrameBlue, part);
          end;
          if (aaAlpha in FFrame.FActions) and (aaAlpha in FNextFrame.FActions) then begin
            Sprite.AnimationAlpha := Interpolate(FFrame.FFrameAlpha, FNextFrame.FFrameAlpha, part);
          end;
          if (aaRotate in FFrame.FActions) and (aaRotate in FNextFrame.FActions) then begin
            Sprite.AnimationRotation := Interpolate(FFrame.FFrameRotation, FNextFrame.FFrameRotation, part);
          end;
          if (aaScale in FFrame.FActions) and (aaScale in FNextFrame.FActions) then begin
            Sprite.AnimationScaleX := Interpolate(FFrame.FFrameScaleX, FNextFrame.FFrameScaleX, part);
            Sprite.AnimationScaleY := Interpolate(FFrame.FFrameScaleY, FNextFrame.FFrameScaleY, part);
          end;
          bDone := True;
        end else begin
          //Move on to next frame
          FFrameTime := FFrameTime - FFrame.FDuration;
          FFrame := FNextFrame;
          FAnimation := FNextAnimation;
          FAnimation.FindNext(FNextFrame, FNextAnimation, FLoopCount, FReturnAnimation);
          //Make sure we're up to date with latest animation progress
          if aaImage in FFrame.FActions then begin
            Sprite.Image := TOmegaAnimationCollection(TOmegaAnimationFrameCollection(FFrame.Collection).FAnimation.Collection).FAnimationList.FOmegaImageList.ImageList.Items[FFrame.FImageListItem];
            Sprite.ImageIndex := FFrame.FNewIndex;
          end;
          if aaPosition in FFrame.FActions then begin
            Sprite.AnimationX := FFrame.FFrameAnimX;
            Sprite.AnimationY := FFrame.FFrameAnimY;
          end;
          if aaColor in FFrame.FActions then begin
            Sprite.Red   := FFrame.FFrameRed;
            Sprite.Green := FFrame.FFrameGreen;
            Sprite.Blue  := FFrame.FFrameBlue;
          end;
          if aaAlpha in FFrame.FActions then begin
            Sprite.AnimationAlpha := FFrame.FFrameAlpha;
          end;
          if aaRotate in FFrame.FActions then begin
            Sprite.AnimationRotation := FFrame.FFrameRotation;
          end;
          if aaScale in FFrame.FActions then begin
            Sprite.AnimationScaleX := FFrame.FFrameScaleX;
            Sprite.AnimationScaleY := FFrame.FFrameScaleY;
          end;
        end;
      end;
      fcStop: begin
        FAnimationStopped := True;
        bDone := True;
      end;
      fcDie: begin
        Sprite.Dead;
        FAnimationStopped := True;
        bDone := True;
      end;
    end;

  until bDone;
end;

procedure TOmegaAnimationPlayer.GosubAnimation(const Value: TOmegaAnimation);
begin
  if FReturnAnimation = nil then
    FReturnAnimation := FAnimation;
  SetAnimation(Value);
end;

procedure TOmegaAnimationPlayer.ResetAnimation;
begin
  FFrame := nil;
  FNextFrame := nil;
  FFrameTime := 0;
  FLoopCount := 0;
  FAnimationStopped := False;
end;

procedure TOmegaAnimationPlayer.SetAnimation(const Value: TOmegaAnimation);
begin
  FAnimation := Value;
  ResetAnimation;
end;

{ TAdvancedAnimationSprite }

constructor TAdvancedAnimationSprite.Create(const AParent: TOmegaSprite);
begin
  inherited Create(AParent);
  FPlayerCount := 0;
  FBaseX := 0;
  FBaseY := 0;
  FBaseRotation := 0;
  FBaseScaleX := 1;
  FBaseScaleY := 1;
  FBaseAlpha := 255;
end;

destructor TAdvancedAnimationSprite.Destroy;
var
  i: Integer;
begin
  for i := FPlayerCount-1 downto 0 do
    if FPlayers[i] <> nil then FPlayers[i].Free;
  inherited;
end;

procedure TAdvancedAnimationSprite.Assign(const Value: TSprite);
var
  i: Integer;
begin
  if Value is TAdvancedAnimationSprite then begin
    if TAdvancedAnimationSprite(Value).FPlayerCount <> FPlayerCount then
      raise Exception.Create('Can not assign TAdvancedAnimationSprite with different playercounts');
    for i := FPlayerCount-1 downto 0 do
      FPlayers[i] := TAdvancedAnimationSprite(Value).FPlayers[i];
  end else
    inherited;
end;

procedure TAdvancedAnimationSprite.Move(const MoveCount: single);
var
  i: Integer;
begin
  for i := 0 to FPlayerCount-1 do
    FPlayers[i].Progress(MoveCount, Self);
end;

function TAdvancedAnimationSprite.GetPlayer(Index: Integer): TOmegaAnimationPlayer;
begin
  Result := FPlayers[Index];
end;

procedure TAdvancedAnimationSprite.SetPlayerCount(const Value: Integer);
var
  i: Integer;
begin
  if FPlayerCount = Value then Exit;
  for i := FPlayerCount-1 downto 0 do
    if FPlayers[i] <> nil then FPlayers[i].Free;
  FPlayerCount := Value;
  SetLength(FPlayers, FPlayerCount);
  for i := FPlayerCount-1 downto 0 do begin
    FPlayers[i] := TOmegaAnimationPlayer.Create;
    FPlayers[i].Animation := nil;
  end;
end;

procedure TAdvancedAnimationSprite.SetPlayer(Index: Integer;const Value: TOmegaAnimationPlayer);
begin
  if Value = FPlayers[Index] then Exit;
  FPlayers[Index].Assign(Value);
end;

procedure TAdvancedAnimationSprite.SetAnimationX(const Value: Single);
begin
  FAnimationX := Value;
  X := FBaseX + FAnimationX;
end;

procedure TAdvancedAnimationSprite.SetAnimationY(const Value: Single);
begin
  FAnimationY := Value;
  Y := FBaseY + FAnimationY;
end;

procedure TAdvancedAnimationSprite.SetAnimationRotation(const Value: Single);
begin
  FAnimationRotation := Value;
  Rotation := FBaseRotation + FAnimationRotation;
end;

procedure TAdvancedAnimationSprite.SetAnimationAlpha(const Value: Integer);
begin
  FAnimationAlpha := Value;
  Alpha := RoundNormal((FBaseAlpha/255) * (FAnimationAlpha/255) * 255);
end;

procedure TAdvancedAnimationSprite.SetAnimationScaleX(const Value: Single);
begin
  FAnimationScaleX := Value;
  ScaleX := FBaseScaleX * FAnimationScaleX;
end;

procedure TAdvancedAnimationSprite.SetAnimationScaleY(const Value: Single);
begin
  FAnimationScaleY := Value;
  ScaleY := FBaseScaleY * FAnimationScaleY;
end;

procedure TAdvancedAnimationSprite.SetBaseX(const Value: Single);
begin
  FBaseX := Value;
  X := FBaseX + FAnimationX;
end;

procedure TAdvancedAnimationSprite.SetBaseY(const Value: Single);
begin
  FBaseY := Value;
  Y := FBaseY + FAnimationY;
end;

procedure TAdvancedAnimationSprite.SetBaseRotation(const Value: Single);
begin
  FBaseRotation := Value;
  Rotation := FBaseRotation + FAnimationRotation;
end;

procedure TAdvancedAnimationSprite.SetBaseAlpha(const Value: Integer);
begin
  FBaseAlpha := Value;
  Alpha := RoundNormal((FBaseAlpha/255) * (FAnimationAlpha/255));
end;

procedure TAdvancedAnimationSprite.SetBaseScaleX(const Value: Single);
begin
  FBaseScaleX := Value;
  ScaleX := RoundNormal(FBaseScaleX * FAnimationScaleX);
end;

procedure TAdvancedAnimationSprite.SetBaseScaleY(const Value: Single);
begin
  FBaseScaleY := Value;
  ScaleY := RoundNormal(FBaseScaleY * FAnimationScaleY);
end;

end.
