{
//****************************************************************
// OmegaFlexiGrid unit
// author : Tobias Fleischer (Tobybear)
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

history:
02/10/04: converted colors to OmegaColor variables
01/26/04: fixed vertex buffer overflow
10/09/03: pre-release
}
unit OmegaFlexiGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, OmegaScreen,
  OmegaImageList, {$I UseD3D8.inc};

type
  T6VertexArray = array[0..5] of TOmegaVertex;
  P6VertexArray = ^T6VertexArray;
  TGridFunction = procedure(x, y: DWORD; p: P6VertexArray);

  TOmegaFlexiGrid = class(TComponent)
  private
    FCol1, FCol2, FCol3, FCol4: TOmegaColor;
    FDoScale: boolean;
    FRotation, rw, rh, rwf, rhf, FScaleX, FScaleY: single;
    FX, FY, FWidth, FHeight: dword;
    FOffsetX, FOffsetY: single;
    FNumberOfNodes, FNumberOfCells, FNumberOfColumns, FNumberOfRows: word;
    Vertices: POmegaVertex;
    FOmegaImageList: TOmegaImageList;
    FOmegaImageIndex: Integer;
    FRotationCenterX, FRotationCenterY: single;
    procedure SetOmegaScreen(const Value: TOmegaScreen);
    procedure SetNumberOfColumns(Value: word);
    procedure SetNumberOfRows(Value: word);
    procedure SetColor(col: TOmegaColor);
    procedure SetOmegaImageList(const Value: TOmegaImageList);
    procedure SetOmegaImageIndex(const Value: Integer);
  protected
    FOmegaScreen: TOmegaScreen;
  public
    GridFunction: TGridFunction;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Draw; overload;
    procedure Draw(_x, _y, _width, _height: dword); overload;
    procedure Init;
    procedure UnInit;
    procedure Calc;
    property NumberOfCells: word read FNumberOfCells;
    property NumberOfNodes: word read FNumberOfNodes;
    property Color: TOmegaColor read FCol1 write SetColor;
    property ColorTL: TOmegaColor read FCol1 write FCol1;
    property ColorTR: TOmegaColor read FCol2 write FCol2;
    property ColorBR: TOmegaColor read FCol3 write FCol3;
    property ColorBL: TOmegaColor read FCol4 write FCol4;
  published
    property ImageIndex: Integer read FOmegaImageIndex write SetOmegaImageIndex;
    property OmegaImageList: TOmegaImageList read FOmegaImageList write SetOmegaImageList;
    property OmegaScreen: TOmegaScreen read FOmegaScreen write SetOmegaScreen;
    property ScaleTexture: boolean read FDoScale write FDoScale;
    property Rotation: single read FRotation write FRotation;
    property RotationCenterX: single read FRotationCenterX write FRotationCenterX;
    property RotationCenterY: single read FRotationCenterY write FRotationCenterY;
    property OffsetX: single read FOffsetX write FOffsetX;
    property OffsetY: single read FOffsetY write FOffsetY;
    property X: dword read FX write FX;
    property Y: dword read FY write FY;
    property NumberOfColumns: word read FNumberOfColumns write SetNumberOfColumns;
    property NumberOfRows: word read FNumberOfRows write SetNumberOfRows;
    property ScaleX: single read FScaleX write FScaleX;
    property ScaleY: single read FScaleY write FScaleY;
    property Width: dword read FWidth write FWidth;
    property Height: dword read FHeight write FHeight;
  end;

implementation

constructor TOmegaFlexiGrid.Create;
begin
 inherited Create(AOwner);
 GridFunction := nil;
 FNumberOfRows := 1;
 FNumberOfColumns := 1;
 FNumberOfCells := 1;
 FNumberOfNodes := 6;
 FWidth := 100;
 FHeight := 100;
 FRotationCenterX := 0.5;
 FRotationCenterY := 0.5;
 FRotation := 0;
 FX := 0;
 FY := 0;
 FOffsetX := 0;
 FOffsetY := 0;
 FScaleX := 1;
 FScaleY := 1;
 FDoScale := true;
 FOmegaImageList := nil;
 FOmegaScreen := nil;
 FOmegaImageIndex := 0;
 SetColor(OmegaColor($FFFFFFFF));
 Vertices := nil;
end;

destructor TOmegaFlexiGrid.Destroy;
begin
 if Vertices <> nil then FreeMem(Vertices);
 inherited Destroy;
end;

procedure TOmegaFlexiGrid.Draw(_x, _y, _width, _height: dword);
begin
 FX := _x;
 FY := _y;
 FWidth := _width;
 FHeight := _height;
 Calc; // later into param functions!
 Draw;
end;

procedure TOmegaFlexiGrid.Draw;
begin
 FOmegaScreen.CopyVertices(PByte(Vertices), FNumberOfNodes);
 FOmegaScreen.Device.SetTexture(0, FOmegaImageList.ImageList.Items[FOmegaImageIndex].image);
 FOmegaScreen.Device.DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2 * FNumberOfCells);
end;

procedure TOmegaFlexiGrid.Calc;
var p, po: POmegaVertex;
    ax, ay, px, py: single;
    //ps1, ps2: single;
    tmp, tmp2: integer;
//    c: td3dcolor;
//var a: integer;
    cx, cy, q, s1, s2, s3, s4: single;
begin
 q := -pi*FRotation/180;
 s1 := FScaleX * cos(q);
 s2 := FScaleX * sin(q);
 s3 := - FScaleY * sin(q);
 s4 := FScaleY * cos(q);

 p := vertices;
 px := Width / FNumberOfColumns;
 py := Height / FNumberOfRows;
 for tmp2 := 0 to FNumberOfRows - 1 do
  for tmp := 0 to FNumberOfColumns - 1 do
  begin
   po := p;
   p^.color := FCol1;
   ax := tmp * px;
   ay := tmp2 * py;
   cx := ax + FRotationCenterX * px;
   cy := ay + FRotationCenterY * py;
   q := s1 * ax + s3 * ay + cx - cx * s1 - cy * s3;
   p^.y := s2 * ax + s4 * ay + cy - cx * s2 - cy * s4; p^.x := q;

   if FDoScale then
   begin
    p^.tu := rwf * ax / Width;
    p^.tv := rhf * ay / Height;
   end else
   begin
    p^.tu := rwf * p^.x / Width;
    p^.tv := rhf * p^.y / Height;
   end;
   p^.x := p^.x + FX + FOffsetX;
   p^.y := p^.y + FY + FOffsetY;
   inc(p);

   p^.color := FCol2;
   ax := tmp * px + px;
   ay := tmp2 * py;
   q := s1 * ax + s3 * ay + cx - cx * s1 - cy * s3;
   p^.y := s2 * ax + s4 * ay + cy - cx * s2 - cy * s4; p^.x := q;

   if FDoScale then
   begin
    p^.tu := rwf * ax / Width;
    p^.tv := rhf * ay / Height;
   end else
   begin
    p^.tu := rwf * p^.x / Width;
    p^.tv := rhf * p^.y / Height;
   end;
   p^.x := p^.x + FX + FOffsetX;
   p^.y := p^.y + FY + FOffsetY;
   inc(p);

   p^.color := FCol4;
   ax := tmp * px;
   ay := tmp2 * py + py;
   q := s1 * ax + s3 * ay + cx - cx * s1 - cy * s3;
   p^.y := s2 * ax + s4 * ay + cy - cx * s2 - cy * s4; p^.x := q;

   if FDoScale then
   begin
    p^.tu := rwf * ax / Width;
    p^.tv := rhf * ay / Height;
   end else
   begin
    p^.tu := rwf * p^.x / Width;
    p^.tv := rhf * p^.y / Height;
   end;
   p^.x := p^.x + FX + FOffsetX;
   p^.y := p^.y + FY + FOffsetY;
   inc(p);

   p^.color := FCol2;
   ax := tmp * px + px;
   ay := tmp2 * py;
   q := s1 * ax + s3 * ay + cx - cx * s1 - cy * s3;
   p^.y := s2 * ax + s4 * ay + cy - cx * s2 - cy * s4; p^.x := q;

   if FDoScale then
   begin
    p^.tu := rwf * ax / Width;
    p^.tv := rhf * ay / Height;
   end else
   begin
    p^.tu := rwf * p^.x / Width;
    p^.tv := rhf * p^.y / Height;
   end;
   p^.x := p^.x + FX + FOffsetX;
   p^.y := p^.y + FY + FOffsetY;
   inc(p);

   p^.color := FCol4;
   ax := tmp * px;
   ay := tmp2 * py + py;
   q := s1 * ax + s3 * ay + cx - cx * s1 - cy * s3;
   p^.y := s2 * ax + s4 * ay + cy - cx * s2 - cy * s4; p^.x := q;

   if FDoScale then
   begin
    p^.tu := rwf * ax / Width;
    p^.tv := rhf * ay / Height;
   end else
   begin
    p^.tu := rwf * p^.x / Width;
    p^.tv := rhf * p^.y / Height;
   end;
   p^.x := p^.x + FX + FOffsetX;
   p^.y := p^.y + FY + FOffsetY;
   inc(p);

   p^.color := FCol3;
   ax := tmp * px + px;
   ay := tmp2 * py + py;
   q := s1 * ax + s3 * ay + cx - cx * s1 - cy * s3;
   p^.y := s2 * ax + s4 * ay + cy - cx * s2 - cy * s4; p^.x := q;

   if FDoScale then
   begin
    p^.tu := rwf * ax / Width;
    p^.tv := rhf * ay / Height;
   end else
   begin
    p^.tu := rwf * p^.x / Width;
    p^.tv := rhf * p^.y / Height;
   end;
   p^.x := p^.x + FX + FOffsetX;
   p^.y := p^.y + FY + FOffsetY;
   inc(p);

   if assigned(GridFunction) then GridFunction(tmp, tmp2, P6VertexArray(po));
  end;
end;

procedure TOmegaFlexiGrid.SetColor(col: TOmegaColor);
begin
 FCol1 := col;
 FCol2 := col;
 FCol3 := col;
 FCol4 := col;
end;

procedure TOmegaFlexiGrid.SetNumberOfColumns(Value: word);
begin
 if Value < 1 then Value := 1;
 FNumberOfColumns := Value;
 Init;
end;

procedure TOmegaFlexiGrid.SetNumberOfRows(Value: word);
begin
 if Value < 1 then Value := 1;
 FNumberOfRows := Value;
 Init;
end;

procedure TOmegaFlexiGrid.SetOmegaScreen(const Value: TOmegaScreen);
begin
 FOmegaScreen := Value;
end;

procedure TOmegaFlexiGrid.SetOmegaImageList(const Value: TOmegaImageList);
begin
 if Value = FOmegaImageList then Exit;
 FOmegaImageList := Value;
 if (FOmegaImageList = nil) then Exit;
 SetOmegaImageIndex(FOmegaImageIndex);
end;

procedure TOmegaFlexiGrid.SetOmegaImageIndex(const Value: Integer);
var a:single;
begin
 if (value < 0) then exit;
 FOmegaImageIndex := Value;
 if (FOmegaImageList = nil) or (FOmegaImageIndex > FOmegaImageList.ImageList.Count-1) then Exit;
 rw := FOmegaImageList.ImageList.Items[FOmegaImageIndex].RealWidth;
 rh := FOmegaImageList.ImageList.Items[FOmegaImageIndex].RealHeight;
 a := FOmegaImageList.ImageList.Items[FOmegaImageIndex].ImageWidth;
 if a = 0 then a := 1;
 rwf := rw / a;
 a := FOmegaImageList.ImageList.Items[FOmegaImageIndex].ImageHeight;
 if a = 0 then a := 1;
 rhf := rh / a;
end;

procedure TOmegaFlexiGrid.Init;
var i: integer;
    pv: POmegaVertex;
begin
 if (FOmegaScreen <> nil) and (FOmegaScreen.Device <> nil) then
 with FOmegaScreen.Device do
 begin
  FNumberOfCells := FNumberOfRows * FNumberOfColumns;
  FNumberOfNodes := FNumberOfCells * 6;
  if FNumberOfNodes >= maxVertices then FNumberOfNodes := maxVertices - 1;
  if Vertices <> nil then
  begin
   FreeMem(Vertices);
   Vertices := nil;
  end;
  GetMem(Vertices, FNumberOfNodes * sizeof(TOmegaVertex));
  pv := Vertices;
  for i := 1 to FNumberOfNodes do
  begin
   pv^.rhw := 1;
   pv^.z := 0;
   pv^.Color := OmegaColor($FFFFFFFF);
   inc(pv);
  end;
 end;
end;

procedure TOmegaFlexiGrid.UnInit;
begin
 if Vertices <> nil then
 begin
  FreeMem(Vertices);
  Vertices := nil;
 end;
end;

end.


