unit OmegaMap;

interface

uses
  Windows, Messages, SysUtils, Classes,OmegaScreen,
  OmegaImageList,dialogs,ExtCtrls,forms, Math,LZRW1,
  OmegaSprite,{$I UseD3D8.inc};

type
  TTriggers = record
    X,Y : Single;
    name : string;
    action : string;
    extra1 : string;
    extra2 : string;
    extra3 : string;
  end;

  TMapLayer = class(TSimpleAnimSprite)
  private
  public
  published
  end;

  TOmegaMap = class(TComponent)
  private
    FWorldX, FWorldY      : single;
    TileWidth,TileHeight : integer;
    OIL : String;
    compress1: Tlzrw1;
    FOmegaScreen : TOmegaScreen;
    FDrawScreenOnly : boolean;
  protected
  public
    DrawSpriteX : single;
    DrawSpriteY : single;
    WidthCount,HeightCount : integer;
    Layer1   : array of array of TMapLayer;
    Layer2   : array of array of TMapLayer;
    Layer3   : array of array of TMapLayer;
    Triggers : array of array of TTriggers;
    ImageList : TOmegaImageList;
    Procedure MoveLeft(MoveCount: single);
    Procedure MoveRight(MoveCount:single);
    Procedure MoveUp(MoveCount:single);
    Procedure MoveDown(MoveCount:single);
    procedure LoadMap(Map : TFileName);
    Procedure DrawLayer1;
    Procedure DrawLayer2;
    Procedure DrawLayer3;
    Procedure Layer1Collision(Sprite : TSprite);
    Procedure Layer2Collision(Sprite : TSprite);
    Procedure Layer3Collision(Sprite : TSprite);
    property  WorldX : single read FWorldX write FWorldX;
    property  WorldY : single read fWorldY write FWorldY;
  published
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    property OmegaScreen : TOmegaScreen read FOmegaScreen write FOmegaScreen;
    property DrawScreenOnly : boolean read FDrawScreenOnly write FDrawScreenOnly default false;
    //property OmegaSprite : TOmegaSprite read FOmegaSprite write FOmegaSprite;
  end;

  var FOmegaSprite : TOmegaSprite;

implementation

constructor TOmegaMap.Create(AOwner : TComponent);
begin
  inherited;
end;

Destructor TOmegaMap.Destroy;
var a,b : integer;
begin
   Compress1.Free;
   Imagelist.Free;

   for a:= 0 to widthcount - 1 do
     for b := 0 to heightcount - 1 do
       begin
         layer1[a,b].Free;
         layer2[a,b].free;
         layer3[a,b].free;
       end;

   FOmegaSprite.Free;
       
   inherited Destroy;
end;

Procedure TOmegaMap.MoveLeft(MoveCount: single);
var
  a,b: integer;
begin
  DrawSpriteX := DrawSpriteX + MoveCount;
  for a := 0 to widthcount-1 do
    for b := 0 to heightcount-1 do
      begin
        Layer1[a,b].X := Layer1[a,b].X - MoveCount;
        Layer2[a,b].X := Layer2[a,b].X - MoveCount;
        Layer3[a,b].X := Layer3[a,b].X - MoveCount;
      end;
end;

Procedure TOmegaMap.MoveRight(MoveCount: single);
var
  a,b: integer;
begin
  DrawSpriteX := DrawSpriteX - MoveCount;
  for a := 0 to widthcount-1 do
    for b := 0 to heightcount-1 do
      begin
        Layer1[a,b].X := Layer1[a,b].X + MoveCount;
        Layer2[a,b].X := Layer2[a,b].X + MoveCount;
        Layer3[a,b].X := Layer3[a,b].X + MoveCount;
      end;
end;

Procedure TOmegaMap.MoveUp(MoveCount: single);
var
  a,b: integer;
begin
  DrawSpriteY := DrawSpriteY + MoveCount;
  for a := 0 to widthcount-1 do
    for b := 0 to heightcount-1 do
      begin
        Layer1[a,b].Y := Layer1[a,b].Y - MoveCount;
        Layer2[a,b].Y := Layer2[a,b].Y - MoveCount;
        Layer3[a,b].Y := Layer3[a,b].Y - MoveCount;
      end;
end;

Procedure TOmegaMap.MoveDown(MoveCount: single);
var
  a,b: integer;
begin
  DrawSpriteY := DrawSpriteY - MoveCount;
  for a := 0 to widthcount-1 do
    for b := 0 to heightcount-1 do
      begin
        Layer1[a,b].Y := Layer1[a,b].Y + MoveCount;
        Layer2[a,b].Y := Layer2[a,b].Y + MoveCount;
        Layer3[a,b].Y := Layer3[a,b].Y + MoveCount;
      end;
end;

procedure TOmegaMap.LoadMap(Map :TFilename);
var
 InputStream, OutputStream: TMemoryStream;
 TheFile: TStringList;
 LineCount: integer;
 a: integer;
 b: integer;
begin
  DrawSpriteX := Application.MainForm.ClientWidth div 2;
  DrawSpriteY := Application.Mainform.ClientHeight div 2;

  Compress1 := Tlzrw1.create(Self);
  Compress1.UseStream := True;
  Outputstream:=TMemoryStream.Create;
  InputStream:=TMemoryStream.Create;
  TheFile:=TStringList.Create;
try
  InputStream.LoadFromFile(map);
  //uncompress file
  compress1.InputStream:=inputstream;
  compress1.OutputStream:=outputstream;
  compress1.deCompress;
  //end uncompress
  outputstream.Position:=0;
  TheFile.LoadFromStream(outputstream);
  linecount:=0;
  //Load map
  //load file part by part

  widthcount := strtoint(TheFile.Strings[linecount]);
  linecount:=linecount+1;

  heightcount := strtoint(TheFile.Strings[linecount]);
  Linecount := linecount+1;

  OIL := TheFile.Strings[linecount];

  FOmegaSprite := TOmegaSprite.Create(self);

  Imagelist := TOmegaImagelist.Create(self);
  ImageList.OmegaScreen := FOmegaScreen;
  ImageList.ImageList.LoadFromFile(extractfilepath(map) + OIL);
  ImageList.Init;

  linecount:=linecount+1;

  TileWidth := strtoint(TheFile.Strings[linecount]);
  linecount:=linecount+1;
  TileHeight := strtoint(TheFile.Strings[linecount]);
  linecount:=linecount+1;

  SetLength(Layer1,(WidthCount),(HeightCount));
  SetLength(Layer2,(WidthCount),(HeightCount));
  SetLength(Layer3,(WidthCount),(HeightCount));
  SetLength(Triggers,(WidthCount),(HeightCount));

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer1[a,b] := TMapLayer.Create(FOmegaSprite);
       Layer1[a,b].Image := Imagelist.ImageList.Items[0];
       Layer1[a,b].Width := TileWidth;
       Layer1[a,b].Height := TileHeight;
     end;

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer2[a,b] := TMapLayer.Create(FOmegaSprite);
       Layer2[a,b].Image := Imagelist.ImageList.Items[0];
       Layer2[a,b].Width := TileWidth;
       Layer2[a,b].Height := TileHeight;
     end;

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer3[a,b] := TMapLayer.Create(FOmegaSprite);
       Layer3[a,b].Image := Imagelist.ImageList.Items[0];
       Layer3[a,b].Width := TileWidth;
       Layer3[a,b].Height := TileHeight;
     end;

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer1[a,b].X :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].Y :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].red :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].green :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].blue :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].alpha :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].rotation :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].image.index := strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].imageindex := strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].animcount := strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].animspeed :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].width :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].height :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer1[a,b].name := TheFile.Strings[linecount];
       linecount:=linecount+1;
       if TheFile.Strings[linecount] = '1' then Layer1[a,b].DoAnimate := true
         else Layer1[a,b].DoAnimate := False;
       linecount:=linecount+1;
     end;

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer2[a,b].X :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].Y :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].red :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].green :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].blue :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].alpha :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].rotation :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].image.index :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].imageindex :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].animcount :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].animspeed :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].width :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].height :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer2[a,b].name := TheFile.Strings[linecount];
       linecount:=linecount+1;
       if TheFile.Strings[linecount] = '1' then Layer2[a,b].DoAnimate := true
         else Layer2[a,b].DoAnimate := False;
       linecount:=linecount+1;
     end;

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer3[a,b].X :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].Y :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].red :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].green :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].blue :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].alpha :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].rotation :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].image.index :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].imageindex :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].animcount :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].animspeed :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       //Layer3[a,b].width :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       //Layer3[a,b].height :=strtoint(TheFile.Strings[linecount]);
       linecount:=linecount+1;
       Layer3[a,b].name := TheFile.Strings[linecount];
       linecount:=linecount+1;
       if TheFile.Strings[linecount] = '1' then Layer3[a,b].DoAnimate := true
         else Layer3[a,b].DoAnimate := False;
       linecount:=linecount+1;
     end;

     for a:=0 to widthcount - 1 do
       for b:=0 to heightcount - 1 do
         begin
           Triggers[a,b].name := TheFile.Strings[linecount];
           linecount:=linecount+1;
           Triggers[a,b].action := TheFile.Strings[linecount];
           linecount:=linecount+1;
           Triggers[a,b].extra1 := TheFile.Strings[linecount];
           linecount:=linecount+1;
           Triggers[a,b].extra2 := TheFile.Strings[linecount];
           linecount:=linecount+1;
           Triggers[a,b].extra3 := TheFile.Strings[linecount];
         end; 

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer1[a,b].Width := TileWidth;
       Layer1[a,b].Height := TileHeight;
     end;

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer2[a,b].Width := TileWidth;
       Layer2[a,b].Height := TileHeight;
     end;

  for a:=0 to widthcount - 1 do
   for b:=0 to heightcount -1 do
     begin
       Layer3[a,b].Width := TileWidth;
       Layer3[a,b].Height := TileHeight;
     end;

 finally
  Inputstream.Free;
  TheFile.Free;
  OutputStream.Free;
 end;
end;

Procedure TOmegaMap.DrawLayer1;
var
  a,b: integer;
  startX,startY : integer;
begin
  Imagelist.BeginDraw;
  if FDrawScreenOnly then
  begin
    StartX := round(DrawSpriteX - Application.MainForm.ClientWidth Div 2) div 32;
    StartY := round(DrawSpriteY - Application.MainForm.ClientHeight Div 2) div 32;

    if StartX < 0 then StartX := 0;
    if StartY < 0 then StartY := 0;
    if StartX + application.mainform.clientwidth div 32  > widthcount-1  then StartX := widthcount-1-application.mainform.clientwidth div 32;
    if StartY + 1 + application.mainform.clientheight div 32 > heightcount-2 then StartY := heightcount-2-application.mainform.clientheight div 32;

    for b := starty to Starty + 1 + Application.MainForm.ClientHeight div 32 do
      for a := startx to Startx + Application.MainForm.ClientWidth div 32 do
        if Layer1[a,b].ImageIndex <> -1 then Layer1[a,b].Draw;
  end
  else
  begin
    for a := 0 to widthcount - 1 do
      for b := 0 to heightcount - 1 do
        if Layer1[a,b].ImageIndex <> -1 then
          Layer1[a,b].Draw;
  end;
  Imagelist.EndDraw;
end;

Procedure TOmegaMap.DrawLayer2;
var
  a,b: integer;
  startX,startY : integer;
begin
  Imagelist.BeginDraw;
  if FDrawScreenOnly then
    begin
      StartX := round(DrawSpriteX - Application.MainForm.ClientWidth Div 2) div 32;
      StartY := round(DrawSpriteY - Application.MainForm.ClientHeight Div 2) div 32;

      if StartX < 0 then StartX := 0;
      if StartY < 0 then StartY := 0;
      if StartX + application.mainform.clientwidth div 32  > widthcount-1  then StartX := widthcount-1-application.mainform.clientwidth div 32;
      if StartY + 1 + application.mainform.clientheight div 32 > heightcount-2 then StartY := heightcount-2-application.mainform.clientheight div 32;

      for b := starty to Starty + 1 + Application.MainForm.ClientHeight div 32 do
        for a := startx to Startx + Application.MainForm.ClientWidth div 32 do
          if Layer2[a,b].ImageIndex <> -1 then Layer2[a,b].Draw;
    end
    else
      begin
        for a := 0 to widthcount -1 do
          for b := 0 to heightcount -1do
            if Layer2[a,b].ImageIndex <> -1 then Layer2[a,b].Draw;
      end;
  ImageList.EndDraw;
end;

Procedure TOmegaMap.DrawLayer3;
var
  a,b: integer;
  startX,startY : integer;
begin
  Imagelist.BeginDraw;
  if FDrawScreenOnly then
    begin
      StartX := round(DrawSpriteX - Application.MainForm.ClientWidth Div 2) div 32;
      StartY := round(DrawSpriteY - Application.MainForm.ClientHeight Div 2) div 32;

      if StartX < 0 then StartX := 0;
      if StartY < 0 then StartY := 0;
      if StartX + application.mainform.clientwidth div 32  > widthcount-1  then StartX := widthcount-1-application.mainform.clientwidth div 32;
      if StartY + 1 + application.mainform.clientheight div 32 > heightcount-2 then StartY := heightcount-2-application.mainform.clientheight div 32;

      for b := starty to Starty + 1 + Application.MainForm.ClientHeight div 32 do
        for a := startx to Startx + Application.MainForm.ClientWidth div 32 do
          if Layer3[a,b].ImageIndex <> -1 then Layer3[a,b].Draw;
    end
    else
      begin
        for a := 0 to widthcount -1 do
          for b := 0 to heightcount -1do
            if Layer3[a,b].ImageIndex <> -1 then Layer3[a,b].Draw;
      end;
  ImageList.EndDraw;
end;

Procedure TOmegaMap.Layer1Collision(Sprite : TSprite);
var
  a,b: integer;
begin
for a := 0 to widthcount -1 do
    for b := 0 to widthcount -1 do
      Layer1[a,b].Collision(Sprite);
end;   

Procedure TOmegaMap.Layer2Collision(Sprite : TSprite);
var
  a,b: integer;
begin
  for a := 0 to widthcount -1 do
    for b := 0 to widthcount -1 do
      Layer2[a,b].Collision(Sprite);
end;

Procedure TOmegaMap.Layer3Collision(Sprite : TSprite);
var
  a,b: integer;
begin
  for a := 0 to widthcount -1 do
    for b := 0 to widthcount -1 do
      Layer3[a,b].Collision(Sprite);
end;
end.
