{
//****************************************************************
// OmegaRenderer unit
// author : Tobias Fleischer (Tobybear)
// contact: tobybear@web.de
// comments are welcome!
//****************************************************************

version history:
07/21/04: added 0.5 constant to SetPositions
09/02/03: now uses new renderer in OmegaScreen
08/31/03: again many new things added
08/12/03: file renamed to OmegaRenderer.pas
          lots of extensions and adaptions
08/05/03: first release
          just to show what can already be done with it

definite "todo":
- add proper documentation
}

unit OmegaRenderer;
interface

uses Windows, Messages, SysUtils, Classes, OmegaScreen,
 OmegaUtils, {$I UseD3D8.inc};

type
  TOmegaRenderer = class
  private
    destroyed: boolean;
    tx1, ty1, tx2, ty2, tx3, ty3, tx4, ty4: single;
    FHorizMirror, FVerticMirror: boolean;
    FDevice: IDirect3DDevice8;
    procedure SetOmegaScreen(const Value: TOmegaScreen);
    procedure SetHMirrorMode(mode: boolean);
    procedure SetVMirrorMode(mode: boolean);
    procedure SetTexture(Texture: IDirect3DTexture8);
  protected
    FOmegaScreen: TOmegaScreen;
  public
    Vertices: array[0..3] of TOmegaVertex;
    constructor Create;
    destructor Destroy; override;
    procedure Init;
    procedure UnInit;
    procedure SetColor(col: TOmegaColor);
    procedure SetColors(col1, col2, col3, col4: TOmegaColor);
    procedure SetPositions(x1, y1, x2, y2, x3, y3, x4, y4: single);
    procedure SetTexturePositions(x1, y1, x2, y2, x3, y3, x4, y4: single);
    procedure Rotate(CenterX, CenterY, Angle: single);
    procedure Scale(CenterX, CenterY, ScaleX, ScaleY: single);
    procedure RotateAndScale(CenterX, CenterY, Angle, ScaleX, ScaleY: single);
    procedure Draw;
    property x1: single read vertices[0].x write vertices[0].x;
    property y1: single read vertices[0].y write vertices[0].y;
    property x2: single read vertices[1].x write vertices[1].x;
    property y2: single read vertices[1].y write vertices[1].y;
    property x3: single read vertices[2].x write vertices[2].x;
    property y3: single read vertices[2].y write vertices[2].y;
    property x4: single read vertices[3].x write vertices[3].x;
    property y4: single read vertices[3].y write vertices[3].y;
    property Texture: IDirect3DTexture8 write SetTexture;
    property Color: TOmegaColor read vertices[0].color write SetColor;
    property ColorTL: TOmegaColor read vertices[0].color write vertices[0].color;
    property ColorTR: TOmegaColor read vertices[1].color write vertices[1].color;
    property ColorBR: TOmegaColor read vertices[2].color write vertices[2].color;
    property ColorBL: TOmegaColor read vertices[3].color write vertices[3].color;
    property OmegaScreen: TOmegaScreen read FOmegaScreen write SetOmegaScreen;
    property MirrorHorizontally: boolean read FHorizMirror write FHorizMirror;
    property MirrorVertically: boolean read FVerticMirror write FVerticMirror;
  end;

implementation

const afac = pi/180;

constructor TOmegaRenderer.Create;
begin
 inherited create;
 vertices[0].z := 0;
 vertices[1].z := 0;
 vertices[2].z := 0;
 vertices[3].z := 0;
 vertices[0].rhw := 1;
 vertices[1].rhw := 1;
 vertices[2].rhw := 1;
 vertices[3].rhw := 1;
 destroyed := false;
end;

destructor TOmegaRenderer.Destroy;
begin
 destroyed := true;
 inherited Destroy;
end;

procedure TOmegaRenderer.Draw;
begin
 FOmegaScreen.CopyVertices(@Vertices, 4);
 FDevice.DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);
end;

procedure TOmegaRenderer.Rotate(CenterX, CenterY, Angle: single);
var a: integer;
    q, s1, s2: single;
begin
 q := Angle*afac;
 s1:= cos(q);
 s2:= sin(q);

 for a := 0 to 3 do
  with Vertices[a] do
  begin
   q := s1*x - s2*y + CenterX - CenterX*s1 + CenterY*s2;
   y := s2*x + s1*y + CenterY - CenterX*s2 - CenterY*s1;
   x := q;
  end;
end;

procedure TOmegaRenderer.RotateAndScale(CenterX, CenterY, Angle, ScaleX, ScaleY: single);
var a: integer;
    q, s1, s2, s3, s4: single;
begin
 q  := Angle * afac;
 s1 := ScaleX * cos(q);
 s2 := ScaleX * sin(q);
 s3 := - ScaleY * sin(q);
 s4 := ScaleY * cos(q);

 for a := 0 to 3 do
  with Vertices[a] do
  begin
   q := s1 * x + s3 * y + CenterX - CenterX * s1 - CenterY * s3;
   y := s2 * x + s4 * y + CenterY - CenterX * s2 - CenterY * s4;
   x := q;
  end;
end;

procedure TOmegaRenderer.Scale(CenterX, CenterY, ScaleX, ScaleY: single);
var a: integer;
begin
 for a := 0 to 3 do
  with Vertices[a] do
  begin
   x := ScaleX * x + CenterX - CenterX * ScaleX;
   y := ScaleY * y + CenterY - CenterY * ScaleY;
  end;
end;

procedure TOmegaRenderer.SetColor(col: TOmegaColor);
begin
 vertices[0].color := col;
 vertices[1].color := col;
 vertices[2].color := col;
 vertices[3].color := col;
end;

procedure TOmegaRenderer.SetHMirrorMode(mode: boolean);
begin
 FHorizMirror  := mode;
 SetTexturePositions(tx1, ty1, tx2, ty2, tx3, ty3, tx4, ty4);
end;

procedure TOmegaRenderer.SetOmegaScreen(const Value: TOmegaScreen);
begin
 FOmegaScreen := Value;
end;

procedure TOmegaRenderer.SetPositions(x1, y1, x2, y2, x3, y3, x4, y4: single);
var a:single;
begin
 a:=0.5;
 vertices[0].x := x1;
 vertices[0].y := y1;
 vertices[1].x := x2+a;
 vertices[1].y := y2;
 vertices[2].x := x3+a;
 vertices[2].y := y3+a;
 vertices[3].x := x4;
 vertices[3].y := y4+a;
end;

procedure TOmegaRenderer.SetTexture(Texture: IDirect3DTexture8);
begin
 FDevice.SetTexture(0, Texture);
end;

procedure TOmegaRenderer.SetTexturePositions(x1, y1, x2, y2, x3, y3, x4,
  y4: single);
var a:single;
begin
 a:=0;
 tx1:= x1;
 ty1:= y1;
 tx2:= x2+a;
 ty2:= y2;
 tx3:= x3+a;
 ty3:= y3+a;
 tx4:= x4;
 ty4:= y4+a;

 if (FHorizMirror) and (FVerticMirror) then
 begin
  vertices[0].tu := tx3;
  vertices[0].tv := ty3;
  vertices[1].tu := tx4;
  vertices[1].tv := ty4;
  vertices[2].tu := tx1;
  vertices[2].tv := ty1;
  vertices[3].tu := tx2;
  vertices[3].tv := ty2;
 end else
 if (FHorizMirror) then
 begin
  vertices[0].tu := tx4;
  vertices[0].tv := ty4;
  vertices[1].tu := tx3;
  vertices[1].tv := ty3;
  vertices[2].tu := tx2;
  vertices[2].tv := ty2;
  vertices[3].tu := tx1;
  vertices[3].tv := ty1;
 end else
 if (FVerticMirror) then
 begin
  vertices[0].tu := tx2;
  vertices[0].tv := ty2;
  vertices[1].tu := tx1;
  vertices[1].tv := ty1;
  vertices[2].tu := tx4;
  vertices[2].tv := ty4;
  vertices[3].tu := tx3;
  vertices[3].tv := ty3;
 end else
 begin
  vertices[0].tu := tx1;
  vertices[0].tv := ty1;
  vertices[1].tu := tx2;
  vertices[1].tv := ty2;
  vertices[2].tu := tx3;
  vertices[2].tv := ty3;
  vertices[3].tu := tx4;
  vertices[3].tv := ty4;
 end;
end;

procedure TOmegaRenderer.SetVMirrorMode(mode: boolean);
begin
 FVerticMirror := mode;
end;

procedure TOmegaRenderer.SetColors(col1, col2, col3, col4: TOmegaColor);
begin
 vertices[0].color := col1;
 vertices[1].color := col2;
 vertices[2].color := col3;
 vertices[3].color := col4;
end;

procedure TOmegaRenderer.Init;
begin
 if (FOmegaScreen <> nil) and (FOmegaScreen.Device <> nil) then
 begin
  FDevice := FOmegaScreen.Device;
  with FDevice do
  begin
   SetHMirrorMode(false);
   SetVMirrorMode(false);
   SetPositions(0, 0, 100, 0, 100, 100, 0, 100);
   SetTexturePositions(0, 0, 1, 0, 1, 1, 0, 1);
   SetColors(OmegaColor($FFFFFFFF), OmegaColor($FFFFFFFF),
   OmegaColor($FFFFFFFF), OmegaColor($FFFFFFFF));
  end;
 end;
end;

procedure TOmegaRenderer.UnInit;
begin
 if destroyed then exit;
end;

end.


