{
//****************************************************************
// OmegaSprite unit
// author : Ztar (original version (up to Omega07))
//          Tobias Fleischer / Tobybear (updates and extensions)
//          Paul Exler / Xeres (updates and extensions)
//          baRman_ru (updates and extensions)
// comments are welcome!
//****************************************************************

version history:
07/25/04: changes by baRman_ru:
 - changed '+ Parent.FWorldX' to '+ Parent.FWorldX' in draw method
 - TSprite.XCenter, TSprite.YCenter added (read-only)
   fast access to (X + Width/2)
 - added SetX(X); SetY(Y); line after image change for updating coordinates (FY3 f.e.).
 - added TAnimSprite.AnimReverse property - animation begin from the last frame
 - added TSprite.Visible property - switch this to True if you don't want 
   the sprite to be Drawn
 - added TSprite.ConstCoords property - switch this to True if you don't want 
   to consider World coordinates
 - GetQuad function added using 'Quadrangle collision code' by Tobybear
   the function extracts real coords of the sprite which depend on rotation
 - function GetSpriteAt(aX, aY: single; aPixelCheck, aOnlyCollisioned: boolean): TSprite;
   this function return a sprite in given coords
   (aX; aY) are coords in TOmegaSprite; aPixelCheck - parameter whether to consider transparent pixels
   aPixelCheck influences only to non-rotated sprites; mirroring are considered w aPixelCheck too
   switch aOnlyCollisioned to true if you want to select only sprites w DoCollision = True
 - TOmegaSprite.PixelTransparent(const aX, aY: integer): boolean;
   do you want to determinate pixel (aX; aY) of the sprite transparent?

02/12/04: added OffsetX and OffsetY properties 
02/01/04: fixed per-pixel sprite collision
01/27/04: fixed sprite collision coordinates
01/19/04: Add/Remove are now declared public
01/18/04: added DrawArea properties
	  only draw sprites in a smaller area if you want
01/17/04: fixed bug in collision coordinates
	  reduced collision calls (thanks Xeres)
01/14/04: rewrote the whole sprite collision routine,
          now also works for mirrored sprites
          parent property is now changeable
          added new OmegaSprite.Change function (thanks Ztar/Patroclus)
11/26/03: added and updated sprite properties,
          made lots of calls easier and removed unnecessary functions
09/23/03: fixed Mirroring
09/19/03: many things added/removed/fixed
09/03/03: DoAdditiveBlend does now work properly
08/22/03: various fixes
08/20/03: updated the texture position settings (added +1)
08/18/03: changed some properties
          Draw method is now virtual again
08/13/03: first extended release

"todo":
- add proper documentation and examples
- add some polygon collision detection methods
}

unit OmegaSprite;

interface

uses
  Windows, Messages, SysUtils, Classes,
  D3DX8, OmegaScreen, OmegaImageList, dialogs,
  ExtCtrls, Math, OmegaUtils, {$I UseD3D8.inc};

type
  TOmegaSprite = class;
  
  TSprite = class(TObject)
  private
    FParent: TOmegaSprite;
    FName: string;

    FX1, FY1, FX2, FY2, FX3, FY3, FX4, FY4: Single;
    FXOffs, FYOffs, FImageIndex: Integer;

    FVisible: boolean;
    FConstCoords: boolean;
    FXCenter, FYCenter: single;

    FBlendMode, FZ: Integer;
    FMirrorV, FMirrorH: boolean;
    FCenterX, FCenterY: single;
    FCol1, FCol2, FCol3, FCol4: TOmegaColor;
    FRed, FGreen, FBlue: Integer;
    FAlpha: Integer;
    FDoCollision: Boolean;
    FDoPixelCheck: Boolean;
    FDoColPosition: Boolean;
    FRotation: Single;
    FImage: TImageListItem;
    FScaleX, FScaleY: Single;
    FOffsetX, FOffsetY: Single;
    FWidth, FHeight: Integer;
    FRect: TRect;
    FDoAdditiveBlend: Boolean;
    FIsDead: Boolean;
    function GetDrawRect: TRect; virtual;
    procedure SetAdditiveBlend(const Value: Boolean);
  protected
    procedure SetName(const value: string); virtual;
    procedure SetRed(const value: Integer); virtual;
    procedure SetGreen(const value: Integer); virtual;
    procedure SetBlue(const value: Integer); virtual;
    procedure SetAlpha(const value: Integer); virtual;
    procedure SetImageIndex(const value: integer); virtual;
    procedure SetX(const Value: Single); virtual;
    procedure SetY(const Value: Single); virtual;
    procedure SetZ(const Value: integer); virtual;
    procedure SetImage(const value: TImageListItem); virtual;
    procedure SetWidth(const Value: integer); virtual;
    procedure SetHeight(const Value: integer); virtual;
  public
    property ColorTL: TOmegaColor read FCol1 write FCol1;
    property ColorTR: TOmegaColor read FCol2 write FCol2;
    property ColorBR: TOmegaColor read FCol3 write FCol3;
    property ColorBL: TOmegaColor read FCol4 write FCol4;
    constructor Create(const AParent: TOmegaSprite); virtual;
    destructor Destroy; override;
    procedure Assign(const Value: TSprite); virtual;
    procedure Collision(const Other: TSprite); virtual;
    procedure Dead; virtual;
    function GetQuad: TQuad;
    function PixelTransparent(const aX, aY: integer): boolean;

    //Override these methods in your own sprite class!
    procedure OnCollision(const Sprite: TSprite; const ColX, ColY: integer); virtual;
    procedure Move(const MoveCount: single); virtual;
    procedure Draw; overload; virtual;

    procedure SetColor(col: TOmegaColor);
    procedure SetColors(col1, col2, col3, col4: TOmegaColor);

    property Visible: boolean read FVisible write FVisible;
    property ConstCoords: boolean read FConstCoords write FConstCoords;
    property XCenter: Single read FXCenter;
    property YCenter: Single read FYCenter;

    property X: Single read FX1 write SetX;
    property Y: Single read FY1 write SetY;
    property Width: integer read FWidth write SetWidth;
    property Height: integer read FHeight write SetHeight;
    property Z: Integer read FZ write SetZ;
    property X1: Single read FX1 write FX1;
    property Y1: Single read FY1 write FY1;
    property X2: Single read FX2 write FX2;
    property Y2: Single read FY2 write FY2;
    property X3: Single read FX3 write FX3;
    property Y3: Single read FY3 write FY3;
    property X4: Single read FX4 write FX4;
    property Y4: Single read FY4 write FY4;
    property ImageIndex: integer read FImageIndex write SetImageIndex;
    property Red: Integer read FRed write SetRed default 255;
    property Green: Integer read FGreen write SetGreen default 255;
    property Blue: Integer read FBlue write SetBlue default 255;
    property Alpha: Integer read FAlpha write SetAlpha default 255;
    property Color: TOmegaColor read FCol1 write SetColor;
    property Rotation: Single read FRotation write FRotation;
    property CenterX: Single read FCenterX write FCenterX;
    property CenterY: Single read FCenterY write FCenterY;
    property ScaleX: Single read FScaleX write FScaleX;
    property ScaleY: Single read FScaleY write FScaleY;
    property OffsetX: Single read FOffsetX write FOffsetX;
    property OffsetY: Single read FOffsetY write FOffsetY;
    property MirrorHorizontally: boolean read FMirrorH write FMirrorH;
    property MirrorVertically: boolean read FMirrorV write FMirrorV;
    property DoAdditiveBlend: Boolean read FDoAdditiveBlend write SetAdditiveBlend default false;
    property BlendMode: integer read FBlendMode write FBlendMode;

    property Name: string read FName write SetName;
    property Image: TImageListItem read FImage write SetImage;
    property DoCollision: boolean read FDoCollision write FDoCollision;
    property DoPixelCheck: boolean read FDoPixelCheck write FDoPixelCheck;
    property DoCollisionPosition: boolean read FDoColPosition write FDoColPosition;
    property Rect: TRect read FRect;
    property Parent: TOmegaSprite read FParent write FParent;
    property IsDead : boolean read FIsDead; // added by Czar
  end;

  TSimpleAnimSprite = class(TSprite)
  private
    FDoAnimate: Boolean;
    FAnimLooped: Boolean;
    FAnimStart: Integer;
    FAnimCount: Integer;
    FAnimReverse: boolean;
    FAnimSpeed: single;
    FAnimPos: single;
    Procedure SetAnimStart(Value: integer);
    Procedure SetAnimReverse(Value: boolean);
  public
    constructor Create(const AParent: TOmegaSprite); override;
    procedure Assign(const Value: TSprite); override;
    Procedure Move(const MoveCount: single); override;
    property AnimPos: single read FAnimPos write FAnimPos;
    property AnimStart: Integer read FAnimStart write SetAnimStart;
    property AnimCount: Integer read FAnimCount write FAnimCount;
    property AnimReverse: Boolean read FAnimReverse write SetAnimReverse;
    property AnimSpeed: single read FAnimSpeed write FAnimSpeed;
    property DoAnimate: Boolean read FDoAnimate write FDoAnimate;
    property AnimLooped: Boolean read FAnimLooped write FAnimLooped;
  end;

  TOmegaSprite = class(TComponent)
  private
    FAreaX1: LongInt;
    FAreaY1: LongInt;
    FAreaX2: LongInt;
    FAreaY2: LongInt;
    FSpriteList: TList;
    FDeadList: TList;
    FWorldX, FWorldY: Single;
    function GetSprite(const index: integer): TSprite;
    function GetCount: integer;
  public
    function GetSpriteAt(aX, aY: single; aPixelCheck, aOnlyCollisioned: boolean): TSprite;
    procedure Add(const Sprite: TSprite);
    procedure Remove(const Sprite: TSprite);
    procedure Change(Sprite: TSprite; dest: TOmegaSprite);
    procedure Move(MoveCount: single);
    procedure Draw;
    procedure Collision;
    procedure Dead;
    property Items[const index: integer]: TSprite read GetSprite; default;
    property Count: integer read GetCount;
    property WorldX: Single read FWorldX write FWorldX;
    property WorldY: Single read FWorldY write FWorldY;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DrawAreaX1: LongInt read FAreaX1 write FAreaX1;
    property DrawAreaY1: LongInt read FAreaY1 write FAreaY1;
    property DrawAreaX2: LongInt read FAreaX2 write FAreaX2;
    property DrawAreaY2: LongInt read FAreaY2 write FAreaY2;
  end;

implementation

constructor TSprite.Create(const AParent: TOmegaSprite);
begin
  inherited Create;
  FParent := AParent;
  FName := '';
  FWidth := 10;
  FHeight := 10;
  FCol1 := OmegaColor($FFFFFFFF);
  FCol2 := OmegaColor($FFFFFFFF);
  FCol3 := OmegaColor($FFFFFFFF);
  FCol4 := OmegaColor($FFFFFFFF);
  FCenterX := 0;//0.5;
  FCenterY := 0;//0.5;
  FX1 := 0;
  FY1 := 0;
  FX2 := 10;
  FY2 := 0;
  FX3 := 10;
  FY3 := 10;
  FX4 := 0;
  FY4 := 10;
  FZ := 0;
  FRed := 255;
  FGreen := 255;
  FBlue := 255;
  FAlpha := 255;
  FImageIndex := 0;
  FRotation := 0;
  FScaleX := 1;
  FScaleY := 1;
  FOffsetX := 0;
  FOffsetY := 0;
  FDoAdditiveBlend := false;
  FMirrorH := false;
  FMirrorV := false;

  FDoCollision := false;
  FDoPixelcheck := false;
  FDoColPosition := false;
  FIsDead := false;
  FBlendMode := bmNormal;
  // bmIgnore; // don't use sprite's blending by default
  FVisible:= true;
  FConstCoords:= false;

  Parent.Add(self);
end;

destructor TSprite.Destroy;
begin
  Parent.Remove(Self);
  Parent.FDeadList.Remove(self);
  inherited;
end;

procedure TSprite.Collision(const Other: TSprite);
var
  InterRect: TRect;
  ColX1, ColX2, ColY1, ColY2: integer;

function FullCollisionTest(const Sprite1, Sprite2: TSprite; var InterRect: TRect): boolean;
var
    Rect1, Rect2, r1, r2: TRect;
    desc1, desc2: TD3DSurfaceDesc;
    LockRect1, LockRect2: TD3DLocked_Rect;
    P1, P2: pointer;
    x, y: integer;
    x1,y1,x2,y2,cx1,cx2,cy1,cy2,w1,w2,l1,l2,t1,t2,h1,h2:integer;
begin
 Result := false;
 if not ((Sprite1.FDoPixelcheck) and (Sprite2.FDoPixelcheck)) then exit;
 Result := true;
 if sprite1 = sprite2 then exit;

 r1 := Sprite1.Rect; l1 := interrect.Left - r1.Left;
 t1 := interrect.Top - r1.Top;
 h1 := r1.Bottom - r1.Top; w1 := r1.Right - r1.Left;

 r2 := Sprite2.Rect; l2 := interrect.Left - r2.Left;
 t2 := interrect.Top - r2.Top;
 h2 := r2.Bottom - r2.Top; w2 := r2.Right - r2.Left;

 if Sprite1.FMirrorV then
 begin
  x1 := w1 - l1 - 1;
  cx1 := -1;
 end else
 begin
  x1 := l1 + 1;
  cx1 := 1;
 end;
 if Sprite1.FMirrorH then
 begin
  y1 := h1 - t1 - 1;
  cy1 := -1;
 end else
 begin
  y1 := t1 + 1;
  cy1 := 1;
 end;

 if Sprite2.FMirrorV then
 begin
  x2 := w2 - l2 - 1;
  cx2 := -1;
 end else
 begin
  x2 := l2 + 1;
  cx2 := 1;
 end;
 if Sprite2.FMirrorH then
 begin
  y2 := h2 - t2 - 1;
  cy2 := -1;
 end else
 begin
  y2 := t2 + 1;
  cy2 := 1;
 end;

 x1 := x1 + Sprite1.FXOffs;
 y1 := y1 + Sprite1.FYOffs;
 x2 := x2 + Sprite2.FXOffs;
 y2 := y2 + Sprite2.FYOffs;

 if (Sprite1.FDoPixelcheck) and (Sprite2.FDoPixelcheck) then
 begin
  Sprite1.Image.Image.GetLevelDesc(0, desc1);
  Sprite2.Image.Image.GetLevelDesc(0, desc2);
  if (desc1.Format <> desc2.Format) then Exit;

  if Sprite1.Image.Image.LockRect(0, LockRect1, nil, D3DLOCK_READONLY) = D3D_OK then
  begin
   if (Sprite2.Image.Image = Sprite1.Image.Image) or (Sprite2.Image.Image.LockRect(0, LockRect2, nil, D3DLOCK_READONLY) = D3D_OK) then
   begin
    if Sprite2.Image.Image = Sprite1.Image.Image then LockRect2 := LockRect1;
    if cardinal(desc1.Format) in [20..22] then //32-bit color depth
    begin
     for y := interrect.Top to interrect.Bottom - 2 do
     begin
      P1 := Pointer(Integer(LockRect1.pBits) +
       y1 * LockRect1.Pitch + x1 * 4);
      P2 := Pointer(Integer(LockRect2.pBits) +
       y2 * LockRect2.Pitch + x2 * 4);
      for x := interrect.Left to interrect.Right - 2 do
      begin
       if (PDWord(P1)^ <> 0) and (PDWord(P2)^ <> 0) then
       begin
        Sprite1.Image.Image.UnlockRect(0);
        Sprite2.Image.Image.UnlockRect(0);
        ColX1 := X - r1.Left;
        ColY1 := Y - r1.Top;
        ColX2 := X - r2.Left;
        ColY2 := Y - r2.Top;
        Exit;
       end;
       inc(PDWord(P1), cx1);
       inc(PDWord(P2), cx2);
      end;
      inc(y1, cy1);
      inc(y2, cy2);
     end;
    end else
    if cardinal(desc1.Format) in [23..26] then //16-bit color depth
    begin
     for y := interrect.Top to interrect.Bottom - 2 do
     begin
      P1 := Pointer(Integer(LockRect1.pBits) +
       y1 * LockRect1.Pitch + x1 * 2);
      P2 := Pointer(Integer(LockRect2.pBits) +
       y2 * LockRect2.Pitch + x2 * 2);
      for x := interrect.Left to interrect.Right - 2 do
      begin
       if (PWord(P1)^ <> 0) and (PWord(P2)^ <> 0) then
       begin
        Sprite1.Image.Image.UnlockRect(0);
        Sprite2.Image.Image.UnlockRect(0);
        ColX1 := ColX1 + X - interrect.left;
        ColY1 := ColY1 + Y - interrect.top;
        ColX2 := ColX2 + X - interrect.left;
        ColY2 := ColY2 + Y - interrect.top;
        Exit;
       end;
       inc(PWord(P1), cx1);
       inc(PWord(P2), cx2);
      end;
      inc(y1, cy1);
      inc(y2, cy2);
     end;
    end;
    Sprite2.Image.Image.UnlockRect(0);
   end;
   Sprite1.Image.Image.UnlockRect(0);
  end;
 end;
 Result := false;
end;

begin
  if (FDoCollision) and (Other.FDoCollision)
  and (not FIsDead) and (not Other.FIsDead) then
  begin
    if windows.IntersectRect(InterRect, FRect, Other.FRect) then
    begin
      if (((FDoPixelcheck) and (Other.FDoPixelCheck)) or ((FDoColPosition or Other.FDoColPosition))) then
      begin
        if FullCollisionTest(Self, Other, InterRect) then
        begin
          OnCollision(Other, ColX1, ColY1);
          Other.OnCollision(Self, ColX2, ColY2);
        end;
      end else
      begin
        OnCollision(Other, 0, 0);
        Other.OnCollision(Self, 0, 0);
      end;
    end;
  end;
end;

procedure TSprite.SetAdditiveBlend(const Value: Boolean);
begin
 FDoAdditiveBlend := Value;
 if FDoAdditiveBlend then FBlendMode := bmSrcAdd else
  FBlendMode := bmNormal;
end;

procedure TSprite.Assign(const Value: TSprite);
begin
  Name := value.Name;
  X1 := value.X1;
  Y1 := value.Y1;
  X2 := value.X2;
  Y2 := value.Y2;
  X3 := value.X3;
  Y3 := value.Y3;
  X4 := value.X4;
  Y4 := value.Y4;
  Z := value.Z;
  Red := value.Red;
  Green := value.Green;
  Blue := value.Blue;
  Alpha := value.Alpha;
  ColorTL := value.ColorTL;
  ColorTR := value.ColorTR;
  ColorBL := value.ColorBL;
  ColorBR := value.ColorBR;
  ImageIndex := value.ImageIndex;
  DoCollision := Value.DoCollision;
  DoPixelcheck := Value.DoPixelCheck;
  DoCollisionPosition := Value.DoCollisionPosition;
  Rotation := value.Rotation;
  Image := value.Image;
  ScaleX := value.ScaleX;
  ScaleY := value.ScaleY;
  OffsetX := value.OffsetX;
  OffsetY := value.OffsetY;
  CenterX := value.CenterX;
  CenterY := value.CenterY;
  MirrorHorizontally := value.MirrorHorizontally;
  MirrorVertically := value.MirrorVertically;
  BlendMode := value.BlendMode;

  Visible := value.Visible;
  ConstCoords := value.ConstCoords;
end;

function TSprite.GetDrawRect: TRect;
var TileHeight, TileWidth, rowcount, colcount: integer;
begin
  Result.Left := 0;
  Result.Right := 0;
  Result.Top := 0;
  Result.Bottom := 0;
  TileHeight := Self.FImage.TileHeight;
  TileWidth := Self.FImage.TileWidth;
  ColCount := Self.FImage.ImageWidth div TileWidth;
  RowCount := Self.FImage.ImageHeight div TileHeight;
  if Self.FImageIndex < 0 then Self.FImageIndex := 0; 
  //Make drawrect point to last rectangle if it is higher
  if Self.FImageIndex >= RowCount * ColCount then
    Self.FImageIndex := RowCount * ColCount - 1;
  Result.Left := (Self.FImageIndex mod ColCount) * TileWidth;
  Result.Right := Result.Left + TileWidth;
  Result.Top := (Self.FImageIndex div ColCount) * TileHeight;
  Result.Bottom := Result.Top + TileHeight;
end;

procedure TSprite.OnCollision(const Sprite: TSprite; const ColX, ColY: integer);
begin
end;

procedure TSprite.Draw;
var
  RelX, RelY: integer;
begin
 if not FVisible then Exit;
 if not assigned(FImage) then exit;
 if FImage.Renderer = nil then
 begin
  messagebox(0,'No OmegaImageList assigned to sprite!','Error',1);
  exit;
 end;

 RelX := RoundNormal(FX1 + FOffsetX);
 RelY := RoundNormal(FY1 + FOffsetY);
 if not FConstCoords then
 begin
   RelX := RoundNormal(RelX - FParent.WorldX);
   RelY := RoundNormal(RelY - FParent.WorldY);
 end;

 if   ((Parent.FAreaX1 = 0) AND (Parent.FAreaY1 = 0) AND
       (Parent.FAreaX2 = 0) AND (Parent.FAreaY2 = 0))
   or ((Parent.FAreaX1 <= RelX + Width) AND
       (Parent.FAreaX2 >= RelX) AND
       (Parent.FAreaY1 <= RelY + Height) AND
       (Parent.FAreaY2 >= RelY)) then
    FImage.Draw4Col(
      RelX, RelY,
      Rotation, FCenterX, FCenterY, FScaleX, FScaleY,
      FCol1, FCol2, FCol3, FCol4, FMirrorH, FMirrorV, FImageIndex, FBlendMode);
end;

procedure TSprite.Move(const MoveCount: single);
begin
end;

procedure TSprite.Dead;
begin
  if not FIsDead then
  begin
    FIsDead := true;
    Parent.FDeadList.Add(self);
  end;
end;

////////////////////////////////////////////////////////////////////////
//                Set Sprite Properties                               //
////////////////////////////////////////////////////////////////////////

procedure TSprite.SetName(const value: string);
begin
  self.FName := Value;
end;

procedure TSprite.SetX;
begin
  FX1 := Value;
  FX2 := Value + FWidth;
  FX3 := FX2;
  FX4 := FX1;
  FRect.Left := RoundNormal(FX1);
  FRect.Right := RoundNormal(FX1) + FWidth;
  FXCenter:= FX1 + FWidth/2;
end;

procedure TSprite.SetY;
begin
  FY1 := Value;
  FY2 := FY1;
  FY3 := Value + FHeight;
  FY4 := FY3;
  FRect.Top := RoundNormal(FY1);
  FRect.Bottom := RoundNormal(FY1) + FHeight;
  FYCenter:= FY1 + FHeight/2;
end;

procedure TSprite.SetZ(const Value: integer);
begin
  if FZ <> Value then
  begin
    FZ := Value;
    Parent.FSpriteList.Remove(self);
    Parent.Add(self);
  end;
end;

procedure TSprite.SetImageIndex(const value: integer);
var ColCount, RowCount: integer;
begin
 self.FImageIndex := Value;
 if FImage = nil then exit;
 ColCount := FImage.ImageWidth  div FImage.TileWidth;
 RowCount := FImage.ImageHeight div FImage.TileHeight;
 FXOffs := (FImageIndex mod ColCount) * FImage.TileWidth;
 FYOffs := (FImageIndex div ColCount) * FImage.TileHeight;
 if Value > RowCount * ColCount then
  self.FImageIndex := RowCount * ColCount;
end;

{
procedure TSprite.SetRotation(const value: single);
begin
 self.FRotation := - value * pi / 180;
end;

function TSprite.GetRotation: Single;
begin
 result := - FRotation * 180 / pi;
end;

procedure TSprite.SetScaleX(const value: single);
begin
  self.FScaleX := Value;
end;

procedure TSprite.SetScaleY(const value: single);
begin
  self.FScaleY := Value;
end;
}

procedure TSprite.SetImage(const value: TImageListItem);
begin
  FImage := value;
  FWidth := value.TileWidth;
  FHeight := value.TileHeight;
  FRect.Right := FRect.Left + FWidth;
  FRect.Bottom := FRect.Top + FHeight;
  SetX(X); SetY(Y);
end;

procedure TSprite.SetWidth(const Value: integer);
begin
  FWidth := Value;
  FRect.Right := FRect.Left + FWidth;
end;

procedure TSprite.SetHeight(const Value: integer);
begin
  FHeight := Value;
  FRect.Bottom := FRect.Top + FHeight
end;

procedure TSprite.SetColors(col1, col2, col3, col4: TOmegaColor);
begin
 FCol1 := col1;
 FCol2 := col2;
 FCol3 := col3;
 FCol4 := col4;
end;

procedure TSprite.SetRed(const value: Integer);
begin
 inherited;
 self.FRed := Value;
 SetColor(OmegaColor(FRed, FGreen, FBlue, FAlpha));
end;

procedure TSprite.SetGreen(const value: Integer);
begin
 inherited;
 self.FGreen := Value;
 SetColor(OmegaColor(FRed, FGreen, FBlue, FAlpha));
end;

procedure TSprite.SetBlue(const value: Integer);
begin
 inherited;
 self.FBlue := Value;
 SetColor(OmegaColor(FRed, FGreen, FBlue, FAlpha));
end;

procedure TSprite.SetAlpha(const value: Integer);
begin
 inherited;
 self.FAlpha := Value;
 SetColor(OmegaColor(FRed, FGreen, FBlue, FAlpha));
end;

procedure TSprite.SetColor(col: TOmegaColor);
begin
 FCol1 := col;
 FCol2 := col;
 FCol3 := col;
 FCol4 := col;
end;

function TSprite.GetQuad: TQuad;
  var w, sx, sy, mx, my: single;
begin
 w   := - rotation * pi / 180;
 mx  := x + width * CenterX;
 my  := y + height * CenterY;
 sx  := x - mx; sy := y - my;
 Result[0].X := Round(cos(w) * sx + sin(w) * sy + mx);
 Result[0].Y := Round(-sin(w) * sx + cos(w) * sy + my);
 sx  := x - mx + width; sy := y - my;
 Result[1].X := Round(cos(w) * sx + sin(w) * sy + mx);
 Result[1].Y := Round(-sin(w) * sx + cos(w) * sy + my);
 sx  := x - mx + width; sy := y - my + height;
 Result[2].X := Round(cos(w) * sx + sin(w) * sy + mx);
 Result[2].Y := Round(-sin(w) * sx + cos(w) * sy + my);
 sx  := x - mx; sy := y - my + height;
 Result[3].X := Round(cos(w) * sx + sin(w) * sy + mx);
 Result[3].Y := Round(-sin(w) * sx + cos(w) * sy + my);
end;

function TSprite.PixelTransparent(const aX, aY: integer): boolean;
var
    sprDesc: TD3DSurfaceDesc;
    sprLockRect: TD3DLocked_Rect;
    P1: pointer;
    targetX, targetY: integer;
begin
  if FMirrorV then targetX:= Width - aX else targetX:= aX;
  if FMirrorH then targetY:= Height - aY else targetY:= aY;

  Result := false;
  Image.Image.GetLevelDesc(0, sprDesc);
  if Image.Image.LockRect(0, sprLockRect, nil, D3DLOCK_READONLY) = D3D_OK then begin
     if Cardinal(sprDesc.Format) in [20..22] then begin
      P1 := Pointer(Integer(sprLockRect.pBits) + (targetY) * sprLockRect.Pitch + (targetX) * 4);
      if (PDWord(P1)^ = 0) then Result := true;
      end;
     if Cardinal(sprDesc.Format) in [23..26] then begin
      P1 := Pointer(Integer(sprLockRect.pBits) + (targetY) * sprLockRect.Pitch + (targetX) * 2);
      if (PWord(P1)^ = 0) then Result := true;
      end;
    end;
  Image.Image.UnlockRect(0);
end;

/////////////////// TOmegaSprite /////////////////////////
constructor TOmegaSprite.Create(AOwner: TComponent);
begin
  inherited;
  FSpriteList := TList.Create;
  FDeadList := TList.Create;
  FWorldX := 0;
  FWorldY := 0;
end;

destructor TOmegaSprite.Destroy;
begin
  while FSpriteList.Count > 0 do
    TSprite(FSpriteList.Items[FSpriteList.Count - 1]).Free;
  FSpriteList.Free;
  FDeadList.Free;
  inherited Destroy;
end;

function TOmegaSprite.GetSpriteAt(aX, aY: single; aPixelCheck, aOnlyCollisioned: boolean): TSprite;
  var i: integer;
begin
  Result := nil;
  for i := FSpriteList.Count - 1 downto 0 do begin
    if aOnlyCollisioned and (not TSprite(FSpriteList.Items[i]).DoCollision) then Continue;
    if Round(TSprite(FSpriteList.Items[i]).Rotation) = 0 then
      if PointInRectangle(aX, aY, TSprite(FSpriteList.Items[i]).FRect) then begin
        if aPixelCheck and TSprite(FSpriteList.Items[i]).PixelTransparent(
         Round(aX - TSprite(FSpriteList.Items[i]).X), 
         Round(aY - TSprite(FSpriteList.Items[i]).Y)) then 
          Continue;
        Result := TSprite(FSpriteList.Items[i]);
        Break;
        end else
    else
      if PointInQuadrangle(aX, aY, TSprite(FSpriteList.Items[i]).GetQuad) then begin
        Result := TSprite(FSpriteList.Items[i]);
        Break;
        end;
    end;
end;

procedure TOmegaSprite.Move(MoveCount: single);
var
  i: integer;
begin
  for i := 0 to FSpriteList.Count - 1 do
    TSprite(FSpriteList.Items[i]).Move(MoveCount);
end;

procedure TOmegaSprite.Dead;
begin
  while FDeadList.Count > 0 do 
    TSprite(FDeadList.Items[FDeadList.Count - 1]).Free;
end;

procedure TOmegaSprite.Draw;
var
  i: integer;
begin
  for i := 0 to FSpriteList.Count - 1 do
    TSprite(FSpriteList.Items[i]).Draw;
end;

procedure TOmegaSprite.Collision;
var
  i, j: integer;
begin
  for i := 0 to FSpriteList.Count - 1 do
    for j := i + 1 to FSpriteList.Count - 1 do
      if (TSprite(FSpriteList.Items[i]).DoCollision) AND 
      (TSprite(FSpriteList.Items[j]).DoCollision) then
       TSprite(FSpriteList.Items[i]).Collision(TSprite(FSpriteList.Items[j]));
end;

procedure TOmegaSprite.Remove(const Sprite: TSprite);
begin
  FSpriteList.Remove(Sprite);
end;

function TOmegaSprite.GetSprite(const index: integer): TSprite;
begin
  if (FSpriteList <> nil) and (index >= 0) and (index < FSpriteList.Count) then
    Result := FSpriteList[index]
  else Result := nil;
end;

function TOmegaSprite.GetCount: integer;
begin
  if FSpriteList <> nil then Result := FSpriteList.Count
  else Result := 0;
end;

procedure TOmegaSprite.Add(const Sprite: TSprite);
var
  l, h, dif, m: integer;
begin
  l := 0;
  h := FSpriteList.Count - 1;
  while l <= h do
  begin
    m := (l + h) div 2;
    dif := TSprite(FSpriteList.Items[m]).FZ - Sprite.FZ;
    if dif < 0 then l := m + 1
    else h := m - 1;
  end;
  FSpriteList.Insert(l, Sprite);
end;

procedure TOmegaSprite.Change(Sprite: TSprite; dest: TOmegaSprite);
begin
  dest.Add(Sprite);
  Sprite.Parent := dest;
  FSpriteList.Remove(Sprite);
end; 

{ TSimpleAnimSprite }

Constructor TSimpleAnimSprite.Create(const AParent: TOmegaSprite);
begin
  inherited;
  FDoAnimate := false;
  FAnimLooped := false;
  FAnimStart := 0;
  FAnimCount := 0;
  FAnimSpeed := 0;
  FAnimPos := 0;
end;

procedure TSimpleAnimSprite.Assign(const Value: TSprite);
begin
  if Value is TSimpleAnimSprite then begin
    DoAnimate := TSimpleAnimSprite(Value).DoAnimate;
    AnimStart := TSimpleAnimSprite(Value).AnimStart;
    AnimCount := TSimpleAnimSprite(Value).AnimCount;
    AnimSpeed := TSimpleAnimSprite(Value).AnimSpeed;
    AnimLooped := TSimpleAnimSprite(Value).AnimLooped;
  end;
  inherited;
end;

Procedure TSimpleAnimSprite.SetAnimStart(Value: integer);
begin
  FAnimStart := value;
  FAnimPos := value;
end;

Procedure TSimpleAnimSprite.SetAnimReverse(Value: boolean);
begin
if FAnimReverse = value then Exit;
FAnimReverse:= value;
if Value then FAnimPos:= FAnimStart + FAnimCount - 1
  else FAnimPos:= FAnimStart;
end;

Procedure TSimpleAnimSprite.Move(const MoveCount: single);
begin
if not FDoAnimate then Exit;
  if FAnimReverse then begin
    FAnimPos := FAnimPos - FAnimSpeed * MoveCount;
    if (FAnimPos < FAnimStart) then
      if FAnimLooped then FAnimPos := FAnimStart + FAnimCount - 1

      else begin
        FAnimPos := FAnimStart;
        FDoAnimate := false;
      end;
    end
  else //if FAnimReverse is FALSE
    begin
    FAnimPos := FAnimPos + FAnimSpeed * MoveCount;
    if (FAnimPos > FAnimStart + FAnimCount - 1) then
    begin
      if FAnimLooped then FAnimPos := FAnimStart
      else begin
        FAnimPos := FAnimStart + FAnimCount - 1;
        FDoAnimate := false;
      end;
    end;
    end;//if FAnimReverse is FALSE
FImageIndex := RoundNormal(FAnimPos);
end; //Procedure TSimpleAnimSprite.Move


end.

