unit PictureEdit;

interface

uses
  Windows, SysUtils, Classes, Forms, Dialogs, Controls, StdCtrls, ExtCtrls,
  ExtDlgs, Menus, Graphics, Clipbrd,pngimage;

type

  {  TDelphiXDIBEditForm  }

  TPictureEditForm = class(TForm)
    LoadButton: TButton;
    SaveButton: TButton;
    ClearButton: TButton;
    OKButton: TButton;
    CancelButton: TButton;
    Panel1: TPanel;
    Bevel1: TBevel;
    lNone: TLabel;
    OpenDialog: TOpenPictureDialog;
    SaveDialog: TSavePictureDialog;
    ImagePreview: TImage;
    lSize: TLabel;
    lClass: TLabel;
    procedure OKButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure LoadButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    FChanged: Boolean;
    procedure UpdateData;
  public
  end;

var
  PictureEditForm: TPictureEditForm;

implementation

{$R *.DFM}

{  TDelphiXDIBEditForm  }

procedure TPictureEditForm.FormShow(Sender: TObject);
begin
  UpdateData;
  CancelButton.SetFocus;
end;

procedure TPictureEditForm.OKButtonClick(Sender: TObject);
begin
  if FChanged then
    ModalResult := mrOK
  else ModalResult := mrCancel;
end;

procedure TPictureEditForm.CancelButtonClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

procedure TPictureEditForm.ClearButtonClick(Sender: TObject);
begin
  FChanged := True;
  ImagePreview.Picture.Graphic := nil;
  UpdateData;
end;

procedure TPictureEditForm.LoadButtonClick(Sender: TObject);
begin
  OpenDialog.Filter := GraphicFilter(TGraphic);
  if OpenDialog.Execute then
  begin
    FChanged := True;
    ImagePreview.Picture.LoadFromFile(OpenDialog.FileName);
  end;
  UpdateData;
end;

procedure TPictureEditForm.SaveButtonClick(Sender: TObject);
begin
  SaveDialog.Filter := GraphicFilter(TGraphicClass(ImagePreview.Picture.Graphic.ClassType));
  SaveDialog.DefaultExt := GraphicExtension(TGraphicClass(ImagePreview.Picture.Graphic.ClassType));

  if SaveDialog.Execute then
    ImagePreview.Picture.SaveToFile(SaveDialog.FileName);
end;

procedure TPictureEditForm.UpdateData;

  procedure Draw2(Width, Height: Integer);
  begin
    ImagePreview.Stretch := True;
    ImagePreview.Left := 6 + -(Width - ImagePreview.Width) div 2;
    ImagePreview.Top := 6 + -(Height - ImagePreview.Height) div 2;
    ImagePreview.Width := Width;
    ImagePreview.Height := Height;
  end;

var
  r, r2: Single;
begin
  if (ImagePreview.Picture.Graphic <> nil) and (not ImagePreview.Picture.Graphic.Empty) and
    (ImagePreview.Picture.Width > 0) and (ImagePreview.Picture.Height > 0) then
  begin
    lSize.Caption := Format('Size: %dx%d', [ImagePreview.Picture.Width, ImagePreview.Picture.Height]);

    lClass.Caption := 'Type: ' + ImagePreview.Picture.Graphic.ClassName;

    lNone.Visible := false;
    ClearButton.Enabled := True;
    SaveButton.Enabled := True;

    ImagePreview.Width := 228;
    ImagePreview.Height := 228;

    if (ImagePreview.Picture.Width > ImagePreview.Width) or (ImagePreview.Picture.Height > ImagePreview.Height) then
    begin
      r := ImagePreview.Width / ImagePreview.Picture.Width;
      r2 := ImagePreview.Height / ImagePreview.Picture.Height;
      if r > r2 then
        r := r2;
      Draw2(Round(r * ImagePreview.Picture.Width), Round(r * ImagePreview.Picture.Height));
    end else
      Draw2(ImagePreview.Picture.Width, ImagePreview.Picture.Height);

  end else
  begin
    lSize.Caption := '';
    lClass.Caption := '';

    ClearButton.Enabled := False;
    lNone.Visible := True;
    SaveButton.Enabled := False;

  end;

  ImagePreview.Invalidate;
end;


end.
